# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/03_models/02_cnn.ipynb.

# %% auto 0
__all__ = ['Conv1D', 'CNN', 'CausalConv1d', 'CConv1D', 'TCN_Block', 'TCN', 'SeperateTCN', 'CRNN', 'SeperateCRNN']

# %% ../../nbs/03_models/02_cnn.ipynb 2
from ..data import *
from .layers import *
from .rnn import *
from fastai.basics import *
from torch.nn.utils.parametrizations import weight_norm

# %% ../../nbs/03_models/02_cnn.ipynb 6
@delegates(nn.Conv1d, keep=True)
def Conv1D(input_size,output_size,kernel_size=3,activation = Mish,wn=True, bn = False, **kwargs):
    conv = nn.Conv1d(input_size,output_size,kernel_size,**kwargs)
    act = activation() if activation is not None else None
    bn = nn.BatchNorm1d(input_size) if bn else None
    m = [m for m in [bn,conv,act] if m is not None]
    return nn.Sequential(*m)

# %% ../../nbs/03_models/02_cnn.ipynb 7
class CNN(nn.Module):
    def __init__(self,input_size,output_size,hl_depth=1,hl_width=10,act = Mish,bn=False):
        super().__init__()
        
        conv_layers = [Conv1D(input_size if i==0 else hl_width,hl_width,bn=bn,activation=act,padding=1)
                                          for i in range(hl_depth)]
        self.conv_layers = nn.Sequential(*conv_layers)
        self.final = nn.Conv1d(hl_width,output_size,kernel_size=1)
        
    def forward(self, x):      
        x_in = x.transpose(1,2)
        out = self.conv_layers(x_in)
        out = self.final(out).transpose(1,2)
        return out

# %% ../../nbs/03_models/02_cnn.ipynb 10
class CausalConv1d(torch.nn.Conv1d):
    def __init__(self,
                 in_channels,
                 out_channels,
                 kernel_size,
                 stride=1,
                 dilation=1,
                 groups=1,
                 bias=True,
                stateful=False):

        super().__init__(
            in_channels,
            out_channels,
            kernel_size=kernel_size,
            stride=stride,
            padding=0,
            dilation=dilation,
            groups=groups,
            bias=bias)
        self.__init_size = (kernel_size - 1) * dilation
        self.x_init = None
        self.stateful = stateful
        
    def forward(self, x):
        if self.x_init is not None and self.x_init.shape[0] != x.shape[0]: 
            self.x_init = None
        
        if self.x_init is None or not self.stateful:
            self.x_init = torch.zeros((x.shape[0],x.shape[1],self.__init_size),device=x.device)
            
        x = torch.cat([self.x_init,x],dim=-1)
        
        out = super().forward(x)
    
        if self.stateful: self.x_init =to_detach( x[...,-self.__init_size:], cpu=False, gather=False) 
            
        return out
    
    def reset_state(self):
        self.x_init = None

# %% ../../nbs/03_models/02_cnn.ipynb 11
@delegates(CausalConv1d, keep=True)
def CConv1D(input_size,output_size,kernel_size=2,activation = Mish,wn=True, bn = False, **kwargs):
    conv = CausalConv1d(input_size,output_size,kernel_size,**kwargs)
    if wn: conv = weight_norm(conv)
    act = activation() if activation is not None else None
    bn = nn.BatchNorm1d(input_size) if bn else None
    m = [m for m in [bn,conv,act] if m is not None]
    return nn.Sequential(*m)

# %% ../../nbs/03_models/02_cnn.ipynb 12
@delegates(CausalConv1d, keep=True)
class TCN_Block(nn.Module):
    def __init__(self,input_size,output_size,num_layers=1,
                 activation = Mish,wn=True, bn = False,stateful=False, **kwargs):
        super().__init__()
        
        layers=[]
        for _ in range(num_layers):
            conv = CausalConv1d(input_size,output_size,2,stateful=stateful,**kwargs)
            if wn: conv = weight_norm(conv)
            act = activation() if activation is not None else None
            bn = nn.BatchNorm1d(input_size) if bn else None
            layers += [m for m in [bn,conv,act] if m is not None]
            
        self.layers = nn.Sequential(*layers)
        
        self.residual = nn.Conv1d(input_size,output_size,kernel_size=1) if output_size!=input_size else None
        
    def forward(self, x):      
        out = self.layers(x)
        out = out + (x if self.residual is None else self.residual(x))  
        return out

# %% ../../nbs/03_models/02_cnn.ipynb 13
class TCN(nn.Module):
    def __init__(self,input_size,output_size,hl_depth=1,hl_width=10,act = Mish,bn=False,stateful=False):
        super().__init__()
        
        conv_layers = [TCN_Block(input_size if i==0 else hl_width,hl_width,
                                      dilation=2**(i),bn=bn,activation=act,stateful=stateful)
                                          for i in range(hl_depth)]
        self.conv_layers = nn.Sequential(*conv_layers)
        self.final = nn.Conv1d(hl_width,output_size,kernel_size=1)
        
    def forward(self, x):      
        x_in = x.transpose(1,2)
        out = self.conv_layers(x_in)
        out = self.final(out).transpose(1,2)
        return out

# %% ../../nbs/03_models/02_cnn.ipynb 15
class SeperateTCN(nn.Module):
    def __init__(self,input_list,output_size,hl_depth=1,hl_width=10,act = Mish,bn=False,stateful=False,final_layer=3):
        super().__init__()
        self.input_list =np.cumsum([0] + input_list)
        
        tcn_width = hl_width//len(input_list)
        layers = [ [TCN_Block(n if i==0 else tcn_width,tcn_width,
                                      dilation=2**(i),bn=bn,activation=act)
                                          for i in range(hl_depth)]
                        for n in input_list]
        self.layers = nn.ModuleList([nn.Sequential(*l) for l in layers])
        
        self.rec_field = (2**hl_depth)-1
        self.final = SeqLinear(tcn_width*len(input_list),output_size,hidden_size=hl_width,hidden_layer=final_layer)
        self.x_init = None
        self.stateful = stateful
        
    def forward(self, x):      
        if self.x_init is not None:
            if self.x_init.shape[0] != x.shape[0]: 
                self.x_init = None
            elif self.stateful: 
                x = torch.cat([self.x_init,x],dim=1)
        
        tcn_out = [layer(x[...,self.input_list[i]:self.input_list[i+1]].transpose(1,2)) 
                   for i,layer in enumerate(self.layers)]
        out = torch.cat(tcn_out,dim=1).transpose(1,2)
        
        out = self.final(out)
        
        if self.stateful:
            if self.x_init is not None: out = out[:,self.rec_field:]
            self.x_init = x[:,-self.rec_field:]
            
        return out
    
    def reset_state(self):
        self.x_init = None

# %% ../../nbs/03_models/02_cnn.ipynb 17
class CRNN(nn.Module):
    def __init__(self,input_size,output_size,num_ft=10,num_cnn_layers=4,num_rnn_layers=2,hs_cnn=10,hs_rnn=10,
         hidden_p=0, input_p=0, weight_p=0, rnn_type='gru',stateful=False):
        super().__init__()
        self.cnn = TCN(input_size,num_ft,num_cnn_layers,hs_cnn,act=nn.ReLU,stateful=stateful)
        self.rnn = SimpleRNN(num_ft,output_size,num_layers=num_rnn_layers,hidden_size=hs_rnn,
                       hidden_p=hidden_p, input_p=input_p, weight_p=weight_p, 
                        rnn_type=rnn_type,stateful=stateful)
        
    def forward(self, x):
        return self.rnn(self.cnn(x))
    

# %% ../../nbs/03_models/02_cnn.ipynb 20
class SeperateCRNN(nn.Module):
    def __init__(self,input_list,output_size,num_ft=10,num_cnn_layers=4,num_rnn_layers=2,hs_cnn=10,hs_rnn=10,
         hidden_p=0, input_p=0, weight_p=0, rnn_type='gru',stateful=False):
        super().__init__()
        self.cnn = SeperateTCN(input_list,num_ft,num_cnn_layers,hs_cnn,act=nn.ReLU,stateful=stateful,final_layer=0)
        self.rnn = SimpleRNN(num_ft,output_size,num_layers=num_rnn_layers,hidden_size=hs_rnn,
                       hidden_p=hidden_p, input_p=input_p, weight_p=weight_p, 
                        rnn_type=rnn_type,stateful=stateful)
        
    def forward(self, x):
        return self.rnn(self.cnn(x))



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/01_quaternions.ipynb.

# %% auto 0
__all__ = ['TensorQuaternionInclination', 'TensorQuaternionAngle', 'rad2deg', 'multiplyQuat', 'norm_quaternion', 'conjQuat',
           'diffQuat', 'relativeQuat', 'safe_acos', 'safe_acos_double', 'inclinationAngle', 'relativeAngle',
           'rollAngle', 'pitchAngle', 'inclinationAngleAbs', 'rand_quat', 'rot_vec', 'quatFromAngleAxis', 'quatInterp',
           'inclination_loss', 'inclination_loss_abs', 'inclination_loss_squared', 'inclination_loss_smooth',
           'abs_inclination', 'ms_inclination', 'rms_inclination', 'smooth_inclination', 'rms_inclination_deg',
           'rms_pitch_deg', 'rms_roll_deg', 'mean_inclination_deg', 'angle_loss', 'angle_loss_opt', 'ms_rel_angle',
           'abs_rel_angle', 'rms_rel_angle_deg', 'mean_rel_angle_deg', 'deg_rmse', 'QuaternionRegularizer',
           'augmentation_groups', 'QuaternionAugmentation', 'Quaternion_ResamplingModel', 'HDF2Quaternion',
           'QuaternionBlock', 'TensorInclination', 'HDF2Inclination', 'InclinationBlock', 'plot_scalar_inclination',
           'plot_quaternion_inclination', 'plot_quaternion_rel_angle', 'show_results', 'show_batch']

# %% ../nbs/01_quaternions.ipynb 2
from .data import *
from .models import *
from .learner import *

from fastai.basics import *
import warnings

# %% ../nbs/01_quaternions.ipynb 4
class TensorQuaternionInclination(TensorSequences): pass
class TensorQuaternionAngle(TensorSequences): pass

# %% ../nbs/01_quaternions.ipynb 10
_pi = torch.Tensor([3.14159265358979323846])
def rad2deg(t):
    return 180. * t / _pi.to(t.device).type(t.dtype)

# %% ../nbs/01_quaternions.ipynb 12
@torch.jit.script
def multiplyQuat(q1, q2):
    """quat1*quat2"""
    o1 = q1[..., 0] * q2[..., 0] - q1[..., 1] * q2[..., 1] - q1[..., 2] * q2[..., 2] - q1[..., 3] * q2[..., 3]
    o2 = q1[..., 0] * q2[..., 1] + q1[..., 1] * q2[..., 0] + q1[..., 2] * q2[..., 3] - q1[..., 3] * q2[..., 2]
    o3 = q1[..., 0] * q2[..., 2] - q1[..., 1] * q2[..., 3] + q1[..., 2] * q2[..., 0] + q1[..., 3] * q2[..., 1]
    o4 = q1[..., 0] * q2[..., 3] + q1[..., 1] * q2[..., 2] - q1[..., 2] * q2[..., 1] + q1[..., 3] * q2[..., 0]
    return torch.stack([o1,o2,o3,o4],dim=-1)

# %% ../nbs/01_quaternions.ipynb 17
def norm_quaternion(q):
    return q / q.norm( p=2,dim=-1)[...,None]

# %% ../nbs/01_quaternions.ipynb 19
_conjugate_quaternion = tensor([1,-1,-1,-1])

def conjQuat(q):
    return q*_conjugate_quaternion.to(q.device).type(q.dtype)

# %% ../nbs/01_quaternions.ipynb 21
def diffQuat(q1,q2,norm=True):
    if norm:
        nq1 = norm_quaternion(q1)
        nq2 = norm_quaternion(q2)
    else:
        nq1 = q1
        nq2 = q2
        
        
    return relativeQuat(nq1,nq2) #somehow relativeQuat does not work for backpropagation
#     return multiplyQuat(nq1, conjQuat(nq2))


@torch.jit.script
def relativeQuat(q1, q2):
    """quat1*inv(quat2)"""
    
    o1 =  q1[..., 0] * q2[..., 0] + q1[..., 1] * q2[..., 1] + q1[..., 2] * q2[..., 2] + q1[..., 3] * q2[..., 3]
    o2 = -q1[..., 0] * q2[..., 1] + q1[..., 1] * q2[..., 0] - q1[..., 2] * q2[..., 3] + q1[..., 3] * q2[..., 2]
    o3 = -q1[..., 0] * q2[..., 2] + q1[..., 1] * q2[..., 3] + q1[..., 2] * q2[..., 0] - q1[..., 3] * q2[..., 1]
    o4 = -q1[..., 0] * q2[..., 3] - q1[..., 1] * q2[..., 2] + q1[..., 2] * q2[..., 1] + q1[..., 3] * q2[..., 0]
    
    return torch.stack([o1,o2,o3,o4],dim=-1)

# %% ../nbs/01_quaternions.ipynb 23
def safe_acos(t,eps = 4e-8):
    '''numericaly stable variant of arcuscosine'''
#     eps = 4e-8 #minimum value for acos(1) != 0
    return t.clamp(-1.0 + eps, 1.0 - eps).acos()

# %% ../nbs/01_quaternions.ipynb 25
def safe_acos_double(t,eps = 1e-16):
    '''numericaly stable variant of arcuscosine, uses 64bit floats for internal computation for increased accuracy and gradient propagation'''
    try:
        # Try to use float64 for higher precision
        return t.type(torch.float64).clamp(-1.0 + eps, 1.0 - eps).acos().type(t.dtype)
    except TypeError as e:
        # If float64 is not supported on this device, warn the user and fall back to float32
        warnings.warn(f"Float64 precision not supported on {t.device} device. Falling back to float32. This may reduce numerical accuracy of quaternion operations. Error: {e}")
        return t.clamp(-1.0 + 1e-6, 1.0 - 1e-6).acos()

# %% ../nbs/01_quaternions.ipynb 27
def inclinationAngle(q1,q2):
    
    q = diffQuat(q1,q2)
    return 2*safe_acos_double((q[..., 3]**2 + q[..., 0]**2).sqrt())

def relativeAngle(q1,q2):
    q = diffQuat(q1,q2)
    return 2*safe_acos_double((q[..., 0]).abs())

# %% ../nbs/01_quaternions.ipynb 29
def rollAngle(q1,q2):
    q = diffQuat(q1,q2)
    w,x,y,z= q[...,0],q[...,1],q[...,2],q[...,3]
    t0 = +2.0 * (w * x + y * z)
    t1 = +1.0 - 2.0 * (x * x + y * y)
    return torch.atan2(t0, t1)

def pitchAngle(q1,q2):
    q = diffQuat(q1,q2)
    w,x,y,z= q[...,0],q[...,1],q[...,2],q[...,3]
    t2 = +2.0 * (w * y - z * x)
    t2 = t2.clamp(-1.0, 1.0 )
    return torch.asin(t2)

# %% ../nbs/01_quaternions.ipynb 31
_unit_quaternion = tensor([1.,0,0,0])
def inclinationAngleAbs(q):
    q = diffQuat(q,_unit_quaternion[None,:])
    return 2*((q[..., 3]**2 + q[..., 0]**2).sqrt()).acos()

# %% ../nbs/01_quaternions.ipynb 33
def rand_quat():
    q = torch.rand((4))*2-1
    q /= q.norm()
    return q

# %% ../nbs/01_quaternions.ipynb 34
def rot_vec(v,q):
    v = F.pad(v, (1,0), "constant", 0)
    return multiplyQuat(conjQuat(q),multiplyQuat(v,q))[...,1:]
#     return multiplyQuat(q,multiplyQuat(v,conjQuat(q)))[...,1:]

# %% ../nbs/01_quaternions.ipynb 37
def quatFromAngleAxis(angle, axis):
    if len(axis.shape) == 2:
        N = max(angle.shape[0], axis.shape[0])
        assert angle.shape in ((1,), (N,))
        assert axis.shape == (N, 3) or axis.shape == (1, 3)
    else:
        N = angle.shape[0]
        assert angle.shape == (N,)
        assert axis.shape == (3,)
        axis = axis[None, :]

    axis = axis/torch.norm(axis, dim=1)[:, None]
    quat = torch.cat([torch.cos(angle/2)[:,None],axis*torch.sin(angle/2)[:, None]],dim=-1)
    return quat

# %% ../nbs/01_quaternions.ipynb 38
def quatInterp(quat, ind, extend=False):
    """
    Interpolates an array of quaternions of (non-integer) indices using Slerp. Sampling indices are in the range
    0..N-1, for values outside of this range, depending on "extend", the first/last element or NaN is returned.

    See also csg_bigdata.dp.utils.vecInterp.

    :param quat: array of input quaternions (N(xB)x4)
    :param ind: vector containing the sampling indices, shape (M,)
    :param extend: if true, the input data is virtually extended by the first/last value
    :return: interpolated quaternions (Mx4)
    """
    N = quat.shape[0]
    M = ind.shape[0]
    assert quat.shape[-1] == 4
    assert ind.shape == (M,)

    ind = ind.to(quat.device)
    ind0 = torch.clamp(torch.floor(ind).type(torch.long), 0, N-1)
    ind1 = torch.clamp(torch.ceil(ind).type(torch.long), 0, N-1)

    q0 = quat[ind0].type(torch.float64)
    q1 = quat[ind1].type(torch.float64)
    q_1_0 = diffQuat(q0, q1)

    # normalize the quaternion for positive w component to ensure
    # that the angle will be [0, 180°]
    invert_sign_ind = q_1_0[..., 0] < 0
    q_1_0[invert_sign_ind] = -q_1_0[invert_sign_ind]

#     angle = 2 * torch.acos(torch.clamp(q_1_0[:, 0], -1, 1))
    angle = 2*safe_acos_double((q_1_0[..., 0]))#.type_as(quat)
    axis = q_1_0[..., 1:]

    # copy over (almost) direct hits
#     with np.errstate(invalid='ignore'):
    direct_ind = angle < 1e-06
    quat_out = torch.empty_like(q0)
    # print(quat_out.shape, direct_ind.shape, q0.shape)
    quat_out[direct_ind] = q0[direct_ind]

    interp_ind = ~direct_ind
    t01 = ind - ind0
    if len(quat.shape) == 3: t01 = t01[:,None] #extend shape if batches are part of the tensor
    q_t_0 = quatFromAngleAxis((t01*angle)[interp_ind], axis[interp_ind])
    quat_out[interp_ind] = multiplyQuat(q0[interp_ind], q_t_0)

    if not extend:
        quat_out[ind < 0] = np.nan
        quat_out[ind > N - 1] = np.nan
#     import pdb;pdb.set_trace()

    
    return quat_out.type_as(quat)

# %% ../nbs/01_quaternions.ipynb 44
def inclination_loss(q1,q2):
    q = diffQuat(q1,q2)
    q_abs = (q[..., 3]**2 + q[..., 0]**2).sqrt()-1
    return (q_abs**2).mean().sqrt()

# %% ../nbs/01_quaternions.ipynb 46
def inclination_loss_abs(q1,q2):
    q = diffQuat(q1,q2)
    q_abs = (q[..., 3]**2 + q[..., 0]**2).sqrt()-1
    return q_abs.abs().mean()

# %% ../nbs/01_quaternions.ipynb 48
def inclination_loss_squared(q1,q2):
    q = diffQuat(q1,q2)
    q_abs = (q[..., 3]**2 + q[..., 0]**2).sqrt()-1
    return (q_abs**2).mean()

# %% ../nbs/01_quaternions.ipynb 50
def inclination_loss_smooth(q1,q2):
    q = diffQuat(q1,q2)
    q_abs = (q[..., 3]**2 + q[..., 0]**2).sqrt()-1
    return F.smooth_l1_loss(q_abs,torch.zeros_like(q_abs))

# %% ../nbs/01_quaternions.ipynb 52
def abs_inclination(q1,q2):
    inclination = inclinationAngle(q1,q2)
    return  inclination.abs().mean()

# %% ../nbs/01_quaternions.ipynb 54
def ms_inclination(q1,q2):
    inclination = inclinationAngle(q1,q2)
    return  (inclination**2).mean()

# %% ../nbs/01_quaternions.ipynb 56
def rms_inclination(q1,q2):
    inclination = inclinationAngle(q1,q2)
    return (inclination**2).mean().sqrt()

# %% ../nbs/01_quaternions.ipynb 58
def smooth_inclination(q1,q2):
    inclination = inclinationAngle(q1,q2)
    return F.smooth_l1_loss(inclination,torch.zeros_like(inclination))

# %% ../nbs/01_quaternions.ipynb 60
def rms_inclination_deg(q1,q2):
    inclination = inclinationAngle(q1,q2)
    return  rad2deg((inclination**2).mean().sqrt())

# %% ../nbs/01_quaternions.ipynb 62
def rms_pitch_deg(q1,q2):
    inclination = pitchAngle(q1,q2)
    return  rad2deg((inclination**2).mean().sqrt())

# %% ../nbs/01_quaternions.ipynb 64
def rms_roll_deg(q1,q2):
    inclination = rollAngle(q1,q2)
    return  rad2deg((inclination**2).mean().sqrt())

# %% ../nbs/01_quaternions.ipynb 66
def mean_inclination_deg(q1,q2):
    inclination = inclinationAngle(q1,q2)
    return  rad2deg(inclination.mean())

# %% ../nbs/01_quaternions.ipynb 68
def angle_loss(q1,q2):
    q = diffQuat(q1,q2)
    return (q[..., 0]-1).abs().mean()

# %% ../nbs/01_quaternions.ipynb 69
def angle_loss_opt(q1,q2):
    q1 = norm_quaternion(q1)
    q2 = norm_quaternion(q2)
    
    q2 = conjQuat(q2)
    q = q1[..., 0] * q2[..., 0] - q1[..., 1] * q2[..., 1] - q1[..., 2] * q2[..., 2] - q1[..., 3] * q2[..., 3]
    return (q-1).abs().mean()

# %% ../nbs/01_quaternions.ipynb 70
def ms_rel_angle(q1,q2):
    rel_angle = relativeAngle(q1,q2)
    return  (rel_angle**2).mean()

# %% ../nbs/01_quaternions.ipynb 72
def abs_rel_angle(q1,q2):
    rel_angle = relativeAngle(q1,q2)
    return  rel_angle.abs().mean()

# %% ../nbs/01_quaternions.ipynb 73
def rms_rel_angle_deg(q1,q2):
    rel_angle = relativeAngle(q1,q2)
    return  rad2deg((rel_angle**2).mean().sqrt())

# %% ../nbs/01_quaternions.ipynb 75
def mean_rel_angle_deg(q1,q2):
    rel_angle = relativeAngle(q1,q2)
    return  rad2deg(rel_angle.mean())

# %% ../nbs/01_quaternions.ipynb 77
def deg_rmse(inp, targ):
    return rad2deg(fun_rmse(inp, targ))

# %% ../nbs/01_quaternions.ipynb 80
from fastai.callback.hook import *
@delegates()
class QuaternionRegularizer(HookCallback):
    "Callback that adds AR and TAR to the loss, calculated by output of provided layer"
    run_before=TrainEvalCallback
    def __init__(self,reg_unit=0.0,detach=False, **kwargs):
        super().__init__(detach=detach,**kwargs)
        store_attr('reg_unit')
        
    def hook(self, m, i, o): 
        if type(o) is torch.Tensor:
            self.out = o
        else:
            self.out = o[0]
    
    def after_loss(self):
        if not self.training: return
        h = self.out.float()
        
        if self.reg_unit != 0.:  
            l_a = float(self.reg_unit) * ((1-h.norm(dim=-1))**2).mean()
#             import pdb; pdb.set_trace()
            self.learn.loss_grad += l_a 

# %% ../nbs/01_quaternions.ipynb 81
def augmentation_groups(u_groups):
    '''returns the rotation list corresponding to the input groups'''
    u_groups = np.cumsum([0] + u_groups)
    return [[u_groups[i],u_groups[i+1]-1] for i in range(len(u_groups)-1)]

# %% ../nbs/01_quaternions.ipynb 83
class QuaternionAugmentation(Transform):
    "A transform that before_call its state at each `__call__`"
    split_idx = 0
    def __init__(self,inp_groups, **kwargs):
        super().__init__(**kwargs)
        self.inp_groups = inp_groups
        self.r_quat = None
        for g in inp_groups: 
            l = g[1]-g[0]+1
            if l != 4 and l != 3: raise AttributeError

    def __call__(self, b, split_idx=None, **kwargs):
        #import pdb; pdb.set_trace()
        self.r_quat = rand_quat()
        return super().__call__(b, split_idx=split_idx, **kwargs) 

    def encodes(self, x:(TensorSequences)):
        #import pdb; pdb.set_trace()
        for g in self.inp_groups:
            tmp = x[...,g[0]:g[1]+1]
            if tmp.shape[-1] == 3:
                x[...,g[0]:g[1]+1] = rot_vec(tmp,self.r_quat)
            else:
                x[...,g[0]:g[1]+1] = multiplyQuat(tmp,self.r_quat)
        return x
    
    def encodes(self, x:(TensorQuaternionInclination,TensorQuaternionAngle)):
        return multiplyQuat(x,self.r_quat)

# %% ../nbs/01_quaternions.ipynb 87
from scipy.signal import resample
class Quaternion_ResamplingModel(nn.Module):
    '''
        Module that resamples the signal before and after the prediction of its model.
        Usefull for using models on datasets with different samplingrates.
        
        sampling_method: method used for resampling ['resample','interpolate']
    '''
    
    def __init__(self,model,fs_targ,fs_mean=0,fs_std=1,quaternion_sampling=True):
        super().__init__()
        self.model =model
        self.fs_targ =fs_targ
        self.register_buffer('fs_mean',tensor(fs_mean))
        self.register_buffer('fs_std',tensor(fs_std))
        self.quaternion_sampling = quaternion_sampling

    def forward(self, x):
        dt = (x[0,0,-1]*self.fs_std)+self.fs_mean
        fs_src = 1/dt
        x_len = x.shape[1]
        res_len = int(x.shape[1]*self.fs_targ/(fs_src+10))
        x_raw = x[...,:-1]
        if x_len == res_len:
            res = self.model(x_raw)
        else:
#             x_new = nn.functional.interpolate(x_raw.transpose(1,2), size=res_len, mode='linear',align_corners=False).transpose(1,2)
#             import pdb;pdb.set_trace()
            x_new = tensor(resample(x_raw.detach().cpu().numpy(),res_len,axis =1)).to(x_raw.device) 
            res = self.model(x_new)
            if self.quaternion_sampling: 
                res = quatInterp(res.transpose(0,1),torch.linspace(0,res.shape[1]-1,x_len)).transpose(0,1)
            else:
                res = nn.functional.interpolate(res.transpose(1,2), size=x_len, mode='linear',align_corners=False).transpose(1,2)
        
        return res

# %% ../nbs/01_quaternions.ipynb 90
def relativeQuat_np(q1, q2):
    """inv(quat1)*quat2"""
    if isinstance(q1, np.ndarray) and q1.shape == (4,):
        q1 = q1[np.newaxis]  # convert to 1x4 matrix
        shape = q2.shape
    elif isinstance(q1, np.ndarray) and q1.shape == (1, 4):
        shape = q2.shape
    elif isinstance(q2, np.ndarray) and q2.shape == (4,):
        q2 = q2[np.newaxis]  # convert to 1x4 matrix
        shape = q1.shape
    elif isinstance(q2, np.ndarray) and q2.shape == (1, 4):
        shape = q1.shape
    else:
        assert q1.shape == q2.shape
        shape = q1.shape
    output = np.zeros(shape=shape)
    output[:, 0] = q1[:, 0] * q2[:, 0] + q1[:, 1] * q2[:, 1] + q1[:, 2] * q2[:, 2] + q1[:, 3] * q2[:, 3]
    output[:, 1] = q1[:, 0] * q2[:, 1] - q1[:, 1] * q2[:, 0] - q1[:, 2] * q2[:, 3] + q1[:, 3] * q2[:, 2]
    output[:, 2] = q1[:, 0] * q2[:, 2] + q1[:, 1] * q2[:, 3] - q1[:, 2] * q2[:, 0] - q1[:, 3] * q2[:, 1]
    output[:, 3] = q1[:, 0] * q2[:, 3] - q1[:, 1] * q2[:, 2] + q1[:, 2] * q2[:, 1] - q1[:, 3] * q2[:, 0]
    return output
def quatFromAngleAxis_np(angle, axis):
    """
        Get quaternion from angle and axis.

        If angle is 0, the output will be an identity quaternion.
        If axis is zero vector, a ValueError will be raised unless corresponding angle is 0.
        If  Nx3 axis or N angle are given, the output will be row-wise broadcasted into Nx4:

        >>> q = csgimu.quatFromAngleAxis([0, 1, 2], [1, 0, 0])
        >>> q
        array([[1.        , 0.        , 0.        , 0.        ],
               [0.87758256, 0.47942554, 0.        , 0.        ],
               [0.54030231, 0.84147098, 0.        , 0.        ]])

        :param angle: scalar or N angle in rad input
        :param axis: Nx3 or 1x3 vector input array
        :param debug: enables debug output
        :param plot: enables debug plot
        :return:
            - output: Nx4 or 1x4 quaternion output array
            - debug: dict with debug values (only if debug==True)

        """

    angle = np.asarray(angle, np.float64)
    axis = np.asarray(axis, np.float64)

    is1D = (angle.shape == tuple() or angle.shape == (1,)) and axis.shape == (3,)

    if angle.shape == tuple():
        angle = angle.reshape(1)  # equivalent to np.atleast_1d
    if axis.shape == (3,):
        axis = axis.reshape((1, 3))

    N = max(angle.shape[0], axis.shape[0])

    # for (1x1) case
    if angle.shape == (1, 1):
        angle = angle.ravel()

    assert angle.shape == (N,) or angle.shape == (1,), f'invalid angle shape: {angle.shape}'
    assert axis.shape == (N, 3) or axis.shape == (1, 3), f'invalid axis shape: {axis.shape}'

    angle_brodcasted = np.broadcast_to(angle, (N,))
    axis_brodcasted = np.broadcast_to(axis, (N, 3))

    norm = np.linalg.norm(axis_brodcasted, axis=1)

    identity = norm < np.finfo(np.float64).eps

    q = np.zeros((N, 4), np.float64)
    q[identity] = np.array([1, 0, 0, 0])
    q[~identity] = np.concatenate((np.cos(angle_brodcasted[~identity][:, np.newaxis] / 2), axis_brodcasted[~identity]
                                 * np.array(np.sin(angle_brodcasted[~identity] / 2.0) / norm[~identity])[:, np.newaxis]),
                                axis=1)

    if is1D:
        q = q.reshape((4,))

    return q

def multiplyQuat_np(q1, q2):
    """quat1*quat2"""
    if isinstance(q1, np.ndarray) and q1.shape == (4,):
        q1 = q1[np.newaxis]  # convert to 1x4 matrix
        shape = q2.shape
    elif isinstance(q1, np.ndarray) and q1.shape == (1, 4):
        shape = q2.shape
    elif isinstance(q2, np.ndarray) and q2.shape == (4,):
        q2 = q2[np.newaxis]  # convert to 1x4 matrix
        shape = q1.shape
    elif isinstance(q2, np.ndarray) and q2.shape == (1, 4):
        shape = q1.shape
    else:
        assert q1.shape == q2.shape
        shape = q1.shape
    output = np.zeros(shape=shape)
    output[:, 0] = q1[:, 0] * q2[:, 0] - q1[:, 1] * q2[:, 1] - q1[:, 2] * q2[:, 2] - q1[:, 3] * q2[:, 3]
    output[:, 1] = q1[:, 0] * q2[:, 1] + q1[:, 1] * q2[:, 0] + q1[:, 2] * q2[:, 3] - q1[:, 3] * q2[:, 2]
    output[:, 2] = q1[:, 0] * q2[:, 2] - q1[:, 1] * q2[:, 3] + q1[:, 2] * q2[:, 0] + q1[:, 3] * q2[:, 1]
    output[:, 3] = q1[:, 0] * q2[:, 3] + q1[:, 1] * q2[:, 2] - q1[:, 2] * q2[:, 1] + q1[:, 3] * q2[:, 0]
    return output


def quatInterp_np(quat, ind, extend=True):
    """
    Interpolates an array of quaternions (Nx4) of (non-integer) indices using Slerp. Sampling indices are in the range
    0..N-1. For values outside of this range, depending on "extend", the first/last element or NaN is returned.
    If the input consists 2 quaternions and ind =[0.5], the result should be (1,4) and be the 50/50 interpolation
    between the two input quaternions.

    >>> a=csgimu.quatInterp(np.array([[1,0,0,0],[0,0,1,0]]), np.array([0.5]), extend=True,debug=False,plot=False)
    >>> a
        array([[0, 0, 0, 0]])

    :param quat: array of input quaternions (Nx4)
    :param ind: vector containing the sampling indices for desired output, shape (M,)
    :param extend: if true, the input data is virtually extended by the first/last value
    :return: interpolated quaternions (Mx4)
    """
    ind = np.atleast_1d(ind)
    N = quat.shape[0]
    M = ind.shape[0]
    assert quat.shape == (N, 4)
    assert ind.shape == (M,)

    ind0 = np.clip(np.floor(ind).astype(int), 0, N-1)
    ind1 = np.clip(np.ceil(ind).astype(int), 0, N-1)

    q0 = quat[ind0]
    q1 = quat[ind1]
    q_1_0 = relativeQuat_np(q0, q1)

    # normalize the quaternion for positive w component to ensure
    # that the angle will be [0, 180°]
    invert_sign_ind = q_1_0[:, 0] < 0
    q_1_0[invert_sign_ind] = -q_1_0[invert_sign_ind]

    angle = 2 * np.arccos(np.clip(q_1_0[:, 0], -1, 1))
    axis = q_1_0[:, 1:]

    # copy over (almost) direct hits
    with np.errstate(invalid='ignore'):
        direct_ind = angle < 1e-06
    quat_out = np.empty_like(q0)
    # print(quat_out.shape, direct_ind.shape, q0.shape)
    quat_out[direct_ind] = q0[direct_ind]

    interp_ind = ~direct_ind
    t01 = ind - ind0
    q_t_0 = quatFromAngleAxis_np((t01*angle)[interp_ind], axis[interp_ind])
    quat_out[interp_ind] = multiplyQuat_np(q0[interp_ind], q_t_0)

    if not extend:
        quat_out[ind < 0] = np.nan
        quat_out[ind > N - 1] = np.nan

    quat_out_norm = np.linalg.norm(quat_out, axis=1)

    return quat_out

# %% ../nbs/01_quaternions.ipynb 91
import h5py


class HDF2Quaternion(HDF2Sequence):

    def _hdf_extract_sequence(self,hdf_path,dataset = None, l_slc = None, r_slc= None, resampling_factor=None, fs_idx =None,dt_idx =False,fast_resample=True):
        '''
        extracts a sequence with the shape [seq_len x num_features]

        hdf_path: file path of hdf file, may be a string or path type
        clms: list of dataset names of sequences in hdf file
        dataset: dataset root for clms. Useful for multiples sequences stored in one file.
        l_slc: left boundary for extraction of a window of the whole sequence
        r_slc: right boundary for extraction of a window of the whole sequence
        resampling_factor: scaling factor for the sequence length, uses 'resample_interp' for resampling
        fs_idx: clms list idx of fs entry in sequence. Will be scaled by resampling_factor after resampling
        dt_idx: clms list idx of dt entry in sequence. Will be scaled by resampling_factor after resampling
        '''

        if resampling_factor is not None:
            seq_len = r_slc-l_slc if l_slc is not None and r_slc is not None else None #calculate seq_len for later slicing, necesary because of rounding errors in resampling
            if l_slc is not None: l_slc= math.floor(l_slc/resampling_factor)
            if r_slc is not None: r_slc= math.ceil(r_slc/resampling_factor)

        with h5py.File(hdf_path,'r') as f:
            ds = f if dataset is None else f[dataset]
            l_array = [(ds[n][l_slc:r_slc]) for n in self.clm_names]
            seq = np.stack(l_array,axis=-1)

        if resampling_factor is not None:
#             res_seq = resample_interp(seq,resampling_factor)
            res_seq = quatInterp_np(seq,np.linspace(0,seq.shape[0]-1,int(seq.shape[0]*resampling_factor)))
            if fs_idx is not None: res_seq[:,fs_idx] = seq[0,fs_idx] * resampling_factor
            if dt_idx is not None: res_seq[:,dt_idx] = seq[0,dt_idx] / resampling_factor
            seq = res_seq

            if seq_len is not None: seq = seq[:seq_len] #cut the part of the sequence that is too long because of resampling rounding errors

        return seq

# %% ../nbs/01_quaternions.ipynb 92
class QuaternionBlock(TransformBlock):
    def __init__(self, seq_extract,padding=False):
        return super().__init__(type_tfms=[seq_extract],
                                batch_tfms=[Normalize(axes=[0,1])],
                                dls_kwargs={} if not padding else {'before_batch': pad_sequence})

    @classmethod
    @delegates(HDF2Quaternion, keep=True)
    def from_hdf(cls, clm_names, seq_cls=TensorQuaternionInclination,padding=False, **kwargs):
        return cls(HDF2Quaternion(clm_names,to_cls=seq_cls,**kwargs), padding)

# %% ../nbs/01_quaternions.ipynb 96
class TensorInclination(TensorSequences): pass

# %% ../nbs/01_quaternions.ipynb 97
import h5py
class HDF2Inclination(HDF2Sequence):
        
    def _hdf_extract_sequence(self,hdf_path,dataset = None, l_slc = None, r_slc= None,down_s=None):
        with h5py.File(hdf_path,'r') as f:
            ds = f if dataset is None else f[dataset]
            l_array = [ds[n][l_slc:r_slc] for n in self.clm_names]
            seq = np.vstack(l_array).T
            seq = array(inclinationAngleAbs(tensor(seq))[:,None])
            return seq

# %% ../nbs/01_quaternions.ipynb 98
class InclinationBlock(TransformBlock):
    def __init__(self, seq_extract,padding=False):
        return super().__init__(type_tfms=[seq_extract],
                                batch_tfms=[Normalize(axes=[0,1])],
                                dls_kwargs={} if not padding else {'before_batch': pad_sequence})

    @classmethod
    @delegates(HDF2Inclination, keep=True)
    def from_hdf(cls, clm_names, seq_cls=TensorInclination,padding=False, **kwargs):
        return cls(HDF2Inclination(clm_names,to_cls=seq_cls,**kwargs), padding)

# %% ../nbs/01_quaternions.ipynb 101
def plot_scalar_inclination(axs,in_sig,targ_sig,out_sig=None, **kwargs):
#     import pdb; pdb.set_trace()
    first_targ = targ_sig[0].repeat(targ_sig.shape[0],1)
    axs[0].plot(rad2deg(targ_sig).detach().numpy())
    axs[0].label_outer()
    axs[0].set_ylabel('inclination[°]')
    
    if out_sig is not None:
        axs[0].plot(rad2deg(out_sig).detach().numpy())
        axs[0].legend(['y','ŷ'])
        axs[1].plot(rad2deg(targ_sig-out_sig).detach().numpy())
        axs[1].label_outer()
        axs[1].set_ylabel('error[°]')
        
    axs[-1].plot(in_sig)

# %% ../nbs/01_quaternions.ipynb 102
def plot_quaternion_inclination(axs,in_sig,targ_sig,out_sig=None, **kwargs):
#     import pdb; pdb.set_trace()
    axs[0].plot(rad2deg(inclinationAngleAbs(targ_sig)).detach().numpy())
    axs[0].label_outer()
    axs[0].legend(['y'])
    axs[0].set_ylabel('inclination[°]')
    
    if out_sig is not None:
        axs[0].plot(rad2deg(inclinationAngleAbs(out_sig)).detach().numpy())
        axs[0].legend(['y','ŷ'])
        axs[1].plot(rad2deg(inclinationAngle(out_sig,targ_sig)).detach().numpy())
        axs[1].label_outer()
        axs[1].set_ylabel('error[°]')
        if 'ref' in kwargs:
#             axs[0].plot(rad2deg(inclinationAngleAbs(kwargs['ref'])))
#             axs[0].legend(['y','ŷ','y_ref'])
            axs[1].plot(rad2deg(inclinationAngle(targ_sig,kwargs['ref'])).detach().numpy())
            axs[1].legend(['ŷ','y_ref'])
        
    axs[-1].plot(in_sig)

# %% ../nbs/01_quaternions.ipynb 103
def plot_quaternion_rel_angle(axs,in_sig,targ_sig,out_sig=None, **kwargs):
#     import pdb; pdb.set_trace()
    first_targ = targ_sig[0].repeat(targ_sig.shape[0],1)
    axs[0].plot(rad2deg(relativeAngle(first_targ,targ_sig)).detach().numpy())
    axs[0].label_outer()
    axs[0].legend(['y'])
    axs[0].set_ylabel('angle[°]')
    
    if out_sig is not None:
        axs[0].plot(rad2deg(relativeAngle(first_targ,out_sig)).detach().numpy())
        axs[0].legend(['y','ŷ'])
        axs[1].plot(rad2deg(relativeAngle(out_sig,targ_sig)).detach().numpy())
        axs[1].label_outer()
        axs[1].set_ylabel('error[°]')
        
    axs[-1].plot(in_sig)

# %% ../nbs/01_quaternions.ipynb 105
@dispatch
def show_results(x:TensorSequences, y:TensorInclination, samples, outs, ctxs=None, max_n=2, **kwargs):
    n_samples = min(len(samples), max_n)
    n_targ = 2
    if n_samples > 3:
        #if there are more then 3 samples to plot then put them in a single figure
        plot_seqs_single_figure(n_samples,n_targ,samples,plot_scalar_inclination,outs, **kwargs)
    else:
        #if there are less then 3 samples to plot then put each in its own figure
        plot_seqs_multi_figures(n_samples,n_targ,samples,plot_scalar_inclination,outs, **kwargs)
    return ctxs

# %% ../nbs/01_quaternions.ipynb 106
@dispatch
def show_batch(x:TensorSequences, y:TensorInclination, samples, ctxs=None, max_n=6, **kwargs):
    n_samples = min(len(samples), max_n)
    n_targ = 1
    if n_samples > 3:
        #if there are more then 3 samples to plot then put them in a single figure
        plot_seqs_single_figure(n_samples,n_targ,samples,plot_scalar_inclination, **kwargs)
    else:
        #if there are less then 3 samples to plot then put each in its own figure
        plot_seqs_multi_figures(n_samples,n_targ,samples,plot_scalar_inclination, **kwargs)
    return ctxs

# %% ../nbs/01_quaternions.ipynb 107
@dispatch
def show_results(x:TensorSequences, y:TensorQuaternionInclination, samples, outs, ctxs=None, max_n=2, **kwargs):
    if 'quat' in kwargs: return show_results(x,TensorSequencesOutput(y), samples,outs, ctxs, max_n , **kwargs)
    n_samples = min(len(samples), max_n)
    n_targ = 2
    if n_samples > 3:
        #if there are more then 3 samples to plot then put them in a single figure
        plot_seqs_single_figure(n_samples,n_targ,samples,plot_quaternion_inclination,outs,**kwargs)
    else:
        #if there are less then 3 samples to plot then put each in its own figure
        plot_seqs_multi_figures(n_samples,n_targ,samples,plot_quaternion_inclination,outs,**kwargs)
    return ctxs

# %% ../nbs/01_quaternions.ipynb 108
@dispatch
def show_batch(x:TensorSequences, y:TensorQuaternionInclination, samples, ctxs=None, max_n=6, **kwargs):
    n_samples = min(len(samples), max_n)
    n_targ = 1
    if n_samples > 3:
        #if there are more then 3 samples to plot then put them in a single figure
        plot_seqs_single_figure(n_samples,n_targ,samples,plot_quaternion_inclination)
    else:
        #if there are less then 3 samples to plot then put each in its own figure
        plot_seqs_multi_figures(n_samples,n_targ,samples,plot_quaternion_inclination)
    return ctxs

# %% ../nbs/01_quaternions.ipynb 109
@dispatch
def show_results(x:TensorSequences, y:TensorQuaternionAngle, samples, outs, ctxs=None, max_n=2, **kwargs):
    n_samples = min(len(samples), max_n)
    n_targ = 2
    if n_samples > 3:
        #if there are more then 3 samples to plot then put them in a single figure
        plot_seqs_single_figure(n_samples,n_targ,samples,plot_quaternion_rel_angle,outs, **kwargs)
    else:
        #if there are less then 3 samples to plot then put each in its own figure
        plot_seqs_multi_figures(n_samples,n_targ,samples,plot_quaternion_rel_angle,outs, **kwargs)
    return ctxs

# %% ../nbs/01_quaternions.ipynb 110
@dispatch
def show_batch(x:TensorSequences, y:TensorQuaternionAngle, samples, ctxs=None, max_n=6, **kwargs):
    n_samples = min(len(samples), max_n)
    n_targ = 1
    if n_samples > 3:
        #if there are more then 3 samples to plot then put them in a single figure
        plot_seqs_single_figure(n_samples,n_targ,samples,plot_quaternion_rel_angle, **kwargs)
    else:
        #if there are less then 3 samples to plot then put each in its own figure
        plot_seqs_multi_figures(n_samples,n_targ,samples,plot_quaternion_rel_angle, **kwargs)
    return ctxs



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/02_learner/03_learner.ipynb.

# %% auto 0
__all__ = ['get_inp_out_size', 'RNNLearner', 'TCNLearner', 'CRNNLearner', 'AR_TCNLearner', 'AR_RNNLearner']

# %% ../../nbs/02_learner/03_learner.ipynb 2
from ..data import *
from ..models import *
from .callbacks import *
from .losses import *

from fastai.basics import *
from fastai.callback.tracker import *

# %% ../../nbs/02_learner/03_learner.ipynb 6
def get_inp_out_size(dls):
    '''returns input and output size of a timeseries databunch'''
    tup = dls.one_batch()
    inp = tup[0].shape[-1]
    out = tup[1].shape[-1]
    return inp,out

# %% ../../nbs/02_learner/03_learner.ipynb 9
@delegates(SimpleRNN, keep=True)
def RNNLearner(dls,loss_func=nn.L1Loss(),metrics=[fun_rmse],n_skip=0,num_layers=1,hidden_size=100,stateful=False,opt_func=Adam,cbs=None,**kwargs):
    if cbs is None: cbs = []

    inp,out = get_inp_out_size(dls)
    model = SimpleRNN(inp,out,num_layers,hidden_size,stateful=stateful,**kwargs)
  
    skip = partial(SkipNLoss,n_skip=n_skip)
        
    metrics= [skip(f) for f in metrics]

    if stateful: 
        cbs.append(TbpttResetCB())
        # if stateful apply n_skip with a callback for the first minibatch of a tbptt sequence
        cbs.append(SkipFirstNCallback(n_skip))
    else:
        loss_func = skip(loss_func)
        
    lrn = Learner(dls,model,loss_func=loss_func,opt_func=opt_func,metrics=metrics,cbs=cbs,lr=3e-3)
    return lrn

# %% ../../nbs/02_learner/03_learner.ipynb 14
@delegates(TCN, keep=True)
def TCNLearner(dls,num_layers=3,hidden_size=100,loss_func=nn.L1Loss(),metrics=[fun_rmse],n_skip=None,opt_func=Adam,cbs=None,**kwargs):
    inp,out = get_inp_out_size(dls)
    n_skip = 2**num_layers if n_skip is None else n_skip
    model = TCN(inp,out,num_layers,hidden_size,**kwargs)
  
    skip = partial(SkipNLoss,n_skip=n_skip)
        
    metrics= [skip(f) for f in metrics]
    loss_func = skip(loss_func)
        
    lrn = Learner(dls,model,loss_func=loss_func,opt_func=opt_func,metrics=metrics,cbs=cbs,lr=3e-3)
    return lrn

# %% ../../nbs/02_learner/03_learner.ipynb 17
@delegates(CRNN, keep=True)
def CRNNLearner(dls,loss_func=nn.L1Loss(),metrics=[fun_rmse],n_skip=0,opt_func=Adam,cbs=None,**kwargs):
    inp,out = get_inp_out_size(dls)
    model = CRNN(inp,out,**kwargs)
  
    skip = partial(SkipNLoss,n_skip=n_skip)
        
    metrics= [skip(f) for f in metrics]
    loss_func = skip(loss_func)
        
    lrn = Learner(dls,model,loss_func=loss_func,opt_func=opt_func,metrics=metrics,cbs=cbs,lr=3e-3)
    return lrn

# %% ../../nbs/02_learner/03_learner.ipynb 20
@delegates(TCN, keep=True)
def AR_TCNLearner(dls,hl_depth=3,alpha=1,beta=1,early_stop=0,metrics=None,n_skip=None,opt_func=Adam,**kwargs):
    n_skip = 2**hl_depth if n_skip is None else n_skip
    skip = partial(SkipNLoss,n_skip=n_skip)
    
    inp,out = get_inp_out_size(dls)
    model = AR_Model(TCN(inp+out,out,hl_depth,**kwargs),ar=False)
    model.init_normalize(dls.one_batch())
    
    cbs=[ARInitCB(),TimeSeriesRegularizer(alpha=alpha,beta=beta,modules=[model.model.conv_layers[-1]])]#SaveModelCallback()
    if early_stop > 0:
        cbs += [EarlyStoppingCallback(patience=early_stop)]
        
    if metrics is None: metrics=SkipNLoss(fun_rmse,n_skip)
        
    lrn = Learner(dls,model,loss_func=nn.L1Loss(),opt_func=opt_func,metrics=metrics,cbs=cbs,lr=3e-3)
    return lrn

# %% ../../nbs/02_learner/03_learner.ipynb 23
@delegates(SimpleRNN, keep=True)
def AR_RNNLearner(dls,alpha=0,beta=0,early_stop=0,metrics=None,n_skip=0,opt_func=Adam,**kwargs):
    skip = partial(SkipNLoss,n_skip=n_skip)
    
    inp,out = get_inp_out_size(dls)
    model = AR_Model(SimpleRNN(inp+out,out,**kwargs),ar=False)
    model.init_normalize(dls.one_batch())
    
    cbs=[ARInitCB(),TimeSeriesRegularizer(alpha=alpha,beta=beta,modules=[model.model.rnn])]#SaveModelCallback()
    if early_stop > 0:
        cbs += [EarlyStoppingCallback(patience=early_stop)]
        
    if metrics is None: metrics=SkipNLoss(fun_rmse,n_skip)
        
    lrn = Learner(dls,model,loss_func=nn.L1Loss(),opt_func=opt_func,metrics=metrics,cbs=cbs,lr=3e-3)
    return lrn



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/01_datasets/01_benchmark.ipynb.

# %% auto 0
__all__ = ['BENCHMARK_DL_KWARGS', 'create_dls_wh', 'create_dls_wh_prediction', 'create_dls_silverbox',
           'create_dls_silverbox_prediction', 'create_dls_cascaded_tanks', 'create_dls_cascaded_tanks_prediction',
           'create_dls_emps', 'create_dls_emps_prediction', 'create_dls_ced', 'create_dls_ced_prediction',
           'create_dls_noisy_wh', 'create_dls_noisy_wh_prediction', 'create_dls_robot_forward',
           'create_dls_robot_forward_prediction', 'create_dls_robot_inverse', 'create_dls_robot_inverse_prediction',
           'create_dls_ship', 'create_dls_ship_prediction', 'create_dls_quad_pelican',
           'create_dls_quad_pelican_prediction', 'create_dls_quad_pi', 'create_dls_quad_pi_prediction',
           'broad_u_imu_acc', 'broad_u_imu_gyr', 'broad_u_imu_mag', 'broad_y_opt_pos', 'broad_y_opt_quat', 'broad_u',
           'create_dls_broad', 'create_dls_broad_prediction', 'external_datasets_simulation',
           'external_datasets_prediction', 'create_dls_from_spec']

# %% ../../nbs/01_datasets/01_benchmark.ipynb 2
from fastai.data.all import *

from ..data import *
from .core import create_dls_downl
import identibench as idb

# %% ../../nbs/01_datasets/01_benchmark.ipynb 3
BENCHMARK_DL_KWARGS = {
    # Simulation Benchmarks
    'BenchmarkWH_Simulation':               {'win_sz': 200},
    'BenchmarkSilverbox_Simulation':        {'win_sz': 200},
    'BenchmarkCascadedTanks_Simulation':   {'win_sz': 150, 'bs': 16},
    'BenchmarkEMPS_Simulation':             {'win_sz': 1000},
    'BenchmarkCED_Simulation':              {'win_sz': 100, 'bs': 16},
    'BenchmarkNoisyWH_Simulation':         {'win_sz': 100, 'stp_sz': 50},
    'BenchmarkRobotForward_Simulation':    {'win_sz': 300, 'valid_stp_sz': 4},
    'BenchmarkRobotInverse_Simulation':    {'win_sz': 300, 'valid_stp_sz': 4},
    'BenchmarkShip_Simulation':             {'win_sz': 100},
    'BenchmarkQuadPelican_Simulation':     {'win_sz': 300, 'valid_stp_sz': 40},
    'BenchmarkQuadPi_Simulation':          {'win_sz': 200, 'valid_stp_sz': 20},

    # Prediction Benchmarks
    'BenchmarkWH_Prediction':               {},
    'BenchmarkSilverbox_Prediction':        {},
    'BenchmarkCascadedTanks_Prediction':   {'bs': 16},
    'BenchmarkEMPS_Prediction':             {},
    'BenchmarkCED_Prediction':              {'bs': 16},
    'BenchmarkNoisyWH_Prediction':         {'stp_sz': 50},
    'BenchmarkRobotForward_Prediction':    {'valid_stp_sz': 4},
    'BenchmarkRobotInverse_Prediction':    {'valid_stp_sz': 4},
    'BenchmarkShip_Prediction':             {},
    'BenchmarkQuadPelican_Prediction':     {'valid_stp_sz': 40},
    'BenchmarkQuadPi_Prediction':          {'valid_stp_sz': 20},
}

# %% ../../nbs/01_datasets/01_benchmark.ipynb 4
@delegates(create_dls_downl,keep=True)
def create_dls_from_spec(
    spec: idb.benchmark.BenchmarkSpecBase, # Specification of the benchmark from identibench
    **kwargs # kwargs for create_dls_downl
    ):
    '''
    Create a dataloaders object from identibench benchmark specification. Extracts
    benchmark specific kwargs from BENCHMARK_DL_KWARGS and adds them to the kwargs for create_dls_downl.
    '''
    # add kwargs form spec to dl_kwargs if the key is not already in dl_kwargs
    spec_kwargs = {
        'u': spec.u_cols,
        'y': spec.y_cols,
        'download_function': spec.download_func,
        'dataset':spec.dataset_path
    }

    #add prediction specific kwargs
    if isinstance(spec, idb.benchmark.BenchmarkSpecPrediction):
        spec_kwargs.update({
            'win_sz': spec.pred_horizon+spec.init_window,
            'valid_stp_sz': spec.pred_step,
            'prediction': True
        })

    #add tsfast specific kwargs using BENCHMARK_DL_KWARGS
    if spec.name in BENCHMARK_DL_KWARGS:
        spec_kwargs.update(BENCHMARK_DL_KWARGS[spec.name])

    dl_kwargs = {**spec_kwargs, **kwargs}
    return create_dls_downl(**dl_kwargs)

# %% ../../nbs/01_datasets/01_benchmark.ipynb 6
create_dls_wh =partial(
    create_dls_from_spec,
    spec=idb.BenchmarkWH_Simulation,
    win_sz=200
)
create_dls_wh_prediction =partial(
    create_dls_from_spec,
    spec=idb.BenchmarkWH_Prediction
)

# %% ../../nbs/01_datasets/01_benchmark.ipynb 10
create_dls_silverbox =partial(
    create_dls_from_spec,
    spec=idb.BenchmarkSilverbox_Simulation,
    win_sz=200
)
create_dls_silverbox_prediction =partial(
    create_dls_from_spec,
    spec=idb.BenchmarkSilverbox_Prediction
)

# %% ../../nbs/01_datasets/01_benchmark.ipynb 13
create_dls_cascaded_tanks =partial(
    create_dls_from_spec,
    spec=idb.BenchmarkCascadedTanks_Simulation,
    win_sz=150,
    bs=16,
)
create_dls_cascaded_tanks_prediction =partial(
    create_dls_from_spec,
    spec=idb.BenchmarkCascadedTanks_Prediction,
    bs=16
)

# %% ../../nbs/01_datasets/01_benchmark.ipynb 17
create_dls_emps =partial(
    create_dls_from_spec,
    spec=idb.BenchmarkEMPS_Simulation,
    win_sz=1000
)
create_dls_emps_prediction =partial(
    create_dls_from_spec,
    spec=idb.BenchmarkEMPS_Prediction
)

# %% ../../nbs/01_datasets/01_benchmark.ipynb 20
create_dls_ced =partial(
    create_dls_from_spec,
    spec=idb.BenchmarkCED_Simulation,
    win_sz=100,
    bs=16
)
create_dls_ced_prediction =partial(
    create_dls_from_spec,
    spec=idb.BenchmarkCED_Prediction,
    bs=16
)

# %% ../../nbs/01_datasets/01_benchmark.ipynb 23
create_dls_noisy_wh =partial(
    create_dls_from_spec,
    spec=idb.BenchmarkNoisyWH_Simulation,
    win_sz=100,
    stp_sz=50
)
create_dls_noisy_wh_prediction =partial(
    create_dls_from_spec,
    spec=idb.BenchmarkNoisyWH_Prediction,
    stp_sz=50
)


# %% ../../nbs/01_datasets/01_benchmark.ipynb 26
create_dls_robot_forward =partial(
    create_dls_from_spec,
    spec=idb.BenchmarkRobotForward_Simulation,
    win_sz=300,
    valid_stp_sz=4
)
create_dls_robot_forward_prediction =partial(
    create_dls_from_spec,
    spec=idb.BenchmarkRobotForward_Prediction,
    valid_stp_sz=4
)

# %% ../../nbs/01_datasets/01_benchmark.ipynb 28
create_dls_robot_inverse =partial(
    create_dls_from_spec,
    spec=idb.BenchmarkRobotInverse_Simulation,
    win_sz=300,
    valid_stp_sz=4
)
create_dls_robot_inverse_prediction =partial(
    create_dls_from_spec,
    spec=idb.BenchmarkRobotInverse_Prediction,
    valid_stp_sz=4
)

# %% ../../nbs/01_datasets/01_benchmark.ipynb 31
create_dls_ship =partial(
    create_dls_from_spec,
    spec=idb.BenchmarkShip_Simulation,
    win_sz=100,
)
create_dls_ship_prediction =partial(
    create_dls_from_spec,
    spec=idb.BenchmarkShip_Prediction,
)

# %% ../../nbs/01_datasets/01_benchmark.ipynb 34
create_dls_quad_pelican =partial(
    create_dls_from_spec,
    spec=idb.BenchmarkQuadPelican_Simulation,
    win_sz=300,
    valid_stp_sz=40
)
create_dls_quad_pelican_prediction =partial(
    create_dls_from_spec,
    spec=idb.BenchmarkQuadPelican_Prediction,
    valid_stp_sz=40
)

# %% ../../nbs/01_datasets/01_benchmark.ipynb 37
create_dls_quad_pi =partial(
    create_dls_from_spec,
    spec=idb.BenchmarkQuadPi_Simulation,
    win_sz=200,
    valid_stp_sz=20,
)
create_dls_quad_pi_prediction =partial(
    create_dls_from_spec,
    spec=idb.BenchmarkQuadPi_Prediction,
    valid_stp_sz=20
)

# %% ../../nbs/01_datasets/01_benchmark.ipynb 40
broad_u_imu_acc = [f'imu_acc{i}' for i in range(3)]
broad_u_imu_gyr = [f'imu_gyr{i}' for i in range(3)]
broad_u_imu_mag = [f'imu_mag{i}' for i in range(3)]
broad_y_opt_pos = [f'opt_pos{i}' for i in range(4)]
broad_y_opt_quat = [f'opt_quat{i}' for i in range(4)]

broad_u = broad_u_imu_acc+broad_u_imu_gyr

# %% ../../nbs/01_datasets/01_benchmark.ipynb 41
#Todo: add spec, when available
create_dls_broad = partial(
    create_dls_downl, 
    download_function=idb.datasets.broad.dl_broad,
    u=broad_u,y=broad_y_opt_quat,
    win_sz=100,
    stp_sz=30,
)

create_dls_broad_prediction = partial(
    create_dls_downl, 
    download_function=idb.datasets.broad.dl_broad,
    u=broad_u,y=broad_y_opt_quat,
    win_sz=100,
    stp_sz=30,
    prediction=True
)

# %% ../../nbs/01_datasets/01_benchmark.ipynb 44
external_datasets_simulation = [
    create_dls_wh,
    create_dls_silverbox,
    create_dls_robot_forward,
    create_dls_noisy_wh,
    create_dls_ced,
    # create_dls_broad,
    create_dls_emps
]

# %% ../../nbs/01_datasets/01_benchmark.ipynb 45
external_datasets_prediction = [
    # create_dls_broad_prediction,
    create_dls_cascaded_tanks_prediction,
    create_dls_emps_prediction,
    create_dls_ced_prediction,
    create_dls_noisy_wh_prediction,
    create_dls_quad_pelican_prediction,
    create_dls_quad_pi_prediction,
    create_dls_robot_forward_prediction,
    create_dls_robot_inverse_prediction,
    create_dls_ship_prediction,
    create_dls_silverbox_prediction,
    create_dls_wh_prediction
]



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/05_benchmark.ipynb.

# %% auto 0
__all__ = ['BENCHMARK_DL_KWARGS', 'create_dls_from_spec']

# %% ../nbs/05_benchmark.ipynb 2
from fastai.data.all import *
from .datasets.core import create_dls_downl
import identibench as idb

# %% ../nbs/05_benchmark.ipynb 3
BENCHMARK_DL_KWARGS = {
    # Simulation Benchmarks
    'BenchmarkWH_Simulation':               {'win_sz': 200},
    'BenchmarkSilverbox_Simulation':        {'win_sz': 200},
    'BenchmarkCascadedTanks_Simulation':   {'win_sz': 150, 'bs': 16},
    'BenchmarkEMPS_Simulation':             {'win_sz': 1000},
    'BenchmarkCED_Simulation':              {'win_sz': 100, 'bs': 16},
    'BenchmarkNoisyWH_Simulation':         {'win_sz': 100, 'stp_sz': 50},
    'BenchmarkRobotForward_Simulation':    {'win_sz': 300, 'valid_stp_sz': 4},
    'BenchmarkRobotInverse_Simulation':    {'win_sz': 300, 'valid_stp_sz': 4},
    'BenchmarkShip_Simulation':             {'win_sz': 100},
    'BenchmarkQuadPelican_Simulation':     {'win_sz': 300, 'valid_stp_sz': 40},
    'BenchmarkQuadPi_Simulation':          {'win_sz': 200, 'valid_stp_sz': 20},

    # Prediction Benchmarks
    'BenchmarkWH_Prediction':               {},
    'BenchmarkSilverbox_Prediction':        {},
    'BenchmarkCascadedTanks_Prediction':   {'bs': 16},
    'BenchmarkEMPS_Prediction':             {},
    'BenchmarkCED_Prediction':              {'bs': 16},
    'BenchmarkNoisyWH_Prediction':         {'stp_sz': 50},
    'BenchmarkRobotForward_Prediction':    {'valid_stp_sz': 4},
    'BenchmarkRobotInverse_Prediction':    {'valid_stp_sz': 4},
    'BenchmarkShip_Prediction':             {},
    'BenchmarkQuadPelican_Prediction':     {'valid_stp_sz': 40},
    'BenchmarkQuadPi_Prediction':          {'valid_stp_sz': 20},
}

# %% ../nbs/05_benchmark.ipynb 4
@delegates(create_dls_downl,keep=True)
def create_dls_from_spec(
    spec: idb.benchmark.BenchmarkSpecBase, # Specification of the benchmark from identibench
    **kwargs # kwargs for create_dls_downl
    ):
    '''
    Create a dataloaders object from identibench benchmark specification. Extracts
    benchmark specific kwargs from BENCHMARK_DL_KWARGS and adds them to the kwargs for create_dls_downl.
    '''
    # add kwargs form spec to dl_kwargs if the key is not already in dl_kwargs
    spec_kwargs = {
        'u': spec.u_cols,
        'y': spec.y_cols,
        'download_function': spec.download_func,
        'dataset':spec.dataset_path
    }

    #add prediction specific kwargs
    if isinstance(spec, idb.benchmark.BenchmarkSpecPrediction):
        spec_kwargs.update({
            'win_sz': spec.pred_horizon+spec.init_window,
            'valid_stp_sz': spec.pred_step,
            'prediction': True
        })

    #add tsfast specific kwargs using BENCHMARK_DL_KWARGS
    if spec.name in BENCHMARK_DL_KWARGS:
        spec_kwargs.update(BENCHMARK_DL_KWARGS[spec.name])

    dl_kwargs = {**spec_kwargs, **kwargs}
    return create_dls_downl(**dl_kwargs)



# Autogenerated by nbdev

d = { 'settings': { 'branch': 'master',
                'doc_baseurl': '/tsfast',
                'doc_host': 'https://daniel-om-weber.github.io',
                'git_url': 'https://github.com/daniel-om-weber/tsfast',
                'lib_path': 'tsfast'},
  'syms': { 'tsfast.basics': {},
            'tsfast.benchmark': {'tsfast.benchmark.create_dls_from_spec': ('benchmark.html#create_dls_from_spec', 'tsfast/benchmark.py')},
            'tsfast.data.block': { 'tsfast.data.block.ScalarBlock': ('00_data/block.html#scalarblock', 'tsfast/data/block.py'),
                                   'tsfast.data.block.ScalarBlock.__init__': ( '00_data/block.html#scalarblock.__init__',
                                                                               'tsfast/data/block.py'),
                                   'tsfast.data.block.ScalarBlock.from_hdf': ( '00_data/block.html#scalarblock.from_hdf',
                                                                               'tsfast/data/block.py'),
                                   'tsfast.data.block.ScalarNormalize': ('00_data/block.html#scalarnormalize', 'tsfast/data/block.py'),
                                   'tsfast.data.block.ScalarNormalize.__init__': ( '00_data/block.html#scalarnormalize.__init__',
                                                                                   'tsfast/data/block.py'),
                                   'tsfast.data.block.ScalarNormalize.decodes': ( '00_data/block.html#scalarnormalize.decodes',
                                                                                  'tsfast/data/block.py'),
                                   'tsfast.data.block.ScalarNormalize.encodes': ( '00_data/block.html#scalarnormalize.encodes',
                                                                                  'tsfast/data/block.py'),
                                   'tsfast.data.block.ScalarNormalize.from_stats': ( '00_data/block.html#scalarnormalize.from_stats',
                                                                                     'tsfast/data/block.py'),
                                   'tsfast.data.block.ScalarNormalize.setups': ( '00_data/block.html#scalarnormalize.setups',
                                                                                 'tsfast/data/block.py'),
                                   'tsfast.data.block.SequenceBlock': ('00_data/block.html#sequenceblock', 'tsfast/data/block.py'),
                                   'tsfast.data.block.SequenceBlock.__init__': ( '00_data/block.html#sequenceblock.__init__',
                                                                                 'tsfast/data/block.py'),
                                   'tsfast.data.block.SequenceBlock.from_hdf': ( '00_data/block.html#sequenceblock.from_hdf',
                                                                                 'tsfast/data/block.py'),
                                   'tsfast.data.block.SequenceBlock.from_numpy': ( '00_data/block.html#sequenceblock.from_numpy',
                                                                                   'tsfast/data/block.py'),
                                   'tsfast.data.block.pad_sequence': ('00_data/block.html#pad_sequence', 'tsfast/data/block.py')},
            'tsfast.data.core': { 'tsfast.data.core.CreateDict': ('00_data/core.html#createdict', 'tsfast/data/core.py'),
                                  'tsfast.data.core.DfApplyFuncSplit': ('00_data/core.html#dfapplyfuncsplit', 'tsfast/data/core.py'),
                                  'tsfast.data.core.DfDropClmExcept': ('00_data/core.html#dfdropclmexcept', 'tsfast/data/core.py'),
                                  'tsfast.data.core.DfFilterQuery': ('00_data/core.html#dffilterquery', 'tsfast/data/core.py'),
                                  'tsfast.data.core.DfHDFCreateWindows': ('00_data/core.html#dfhdfcreatewindows', 'tsfast/data/core.py'),
                                  'tsfast.data.core.DfHDFGetSeqLen': ('00_data/core.html#dfhdfgetseqlen', 'tsfast/data/core.py'),
                                  'tsfast.data.core.DfResamplingFactor': ('00_data/core.html#dfresamplingfactor', 'tsfast/data/core.py'),
                                  'tsfast.data.core.FilterClm': ('00_data/core.html#filterclm', 'tsfast/data/core.py'),
                                  'tsfast.data.core.HDF2Scalars': ('00_data/core.html#hdf2scalars', 'tsfast/data/core.py'),
                                  'tsfast.data.core.HDF2Scalars.__init__': ( '00_data/core.html#hdf2scalars.__init__',
                                                                             'tsfast/data/core.py'),
                                  'tsfast.data.core.HDF2Scalars._extract_dict_scalars': ( '00_data/core.html#hdf2scalars._extract_dict_scalars',
                                                                                          'tsfast/data/core.py'),
                                  'tsfast.data.core.HDF2Scalars.encodes': ('00_data/core.html#hdf2scalars.encodes', 'tsfast/data/core.py'),
                                  'tsfast.data.core.HDF2Sequence': ('00_data/core.html#hdf2sequence', 'tsfast/data/core.py'),
                                  'tsfast.data.core.HDF2Sequence.__init__': ( '00_data/core.html#hdf2sequence.__init__',
                                                                              'tsfast/data/core.py'),
                                  'tsfast.data.core.HDF2Sequence._extract_dict_sequence': ( '00_data/core.html#hdf2sequence._extract_dict_sequence',
                                                                                            'tsfast/data/core.py'),
                                  'tsfast.data.core.HDF2Sequence._hdf_extract_sequence': ( '00_data/core.html#hdf2sequence._hdf_extract_sequence',
                                                                                           'tsfast/data/core.py'),
                                  'tsfast.data.core.HDF2Sequence.encodes': ( '00_data/core.html#hdf2sequence.encodes',
                                                                             'tsfast/data/core.py'),
                                  'tsfast.data.core.Memoize': ('00_data/core.html#memoize', 'tsfast/data/core.py'),
                                  'tsfast.data.core.Memoize.__call__': ('00_data/core.html#memoize.__call__', 'tsfast/data/core.py'),
                                  'tsfast.data.core.Memoize.__init__': ('00_data/core.html#memoize.__init__', 'tsfast/data/core.py'),
                                  'tsfast.data.core.MemoizeMP': ('00_data/core.html#memoizemp', 'tsfast/data/core.py'),
                                  'tsfast.data.core.MemoizeMP.__call__': ('00_data/core.html#memoizemp.__call__', 'tsfast/data/core.py'),
                                  'tsfast.data.core.MemoizeMP.__del__': ('00_data/core.html#memoizemp.__del__', 'tsfast/data/core.py'),
                                  'tsfast.data.core.MemoizeMP.__init__': ('00_data/core.html#memoizemp.__init__', 'tsfast/data/core.py'),
                                  'tsfast.data.core.MemoizeMP.cleanup_shared_memory': ( '00_data/core.html#memoizemp.cleanup_shared_memory',
                                                                                        'tsfast/data/core.py'),
                                  'tsfast.data.core.ScalarSequenceElement': ( '00_data/core.html#scalarsequenceelement',
                                                                              'tsfast/data/core.py'),
                                  'tsfast.data.core.ScalarSequenceElement.__init__': ( '00_data/core.html#scalarsequenceelement.__init__',
                                                                                       'tsfast/data/core.py'),
                                  'tsfast.data.core.ScalarSequenceElement.encodes': ( '00_data/core.html#scalarsequenceelement.encodes',
                                                                                      'tsfast/data/core.py'),
                                  'tsfast.data.core.TensorScalars': ('00_data/core.html#tensorscalars', 'tsfast/data/core.py'),
                                  'tsfast.data.core.TensorScalars.from_hdf': ( '00_data/core.html#tensorscalars.from_hdf',
                                                                               'tsfast/data/core.py'),
                                  'tsfast.data.core.TensorScalarsInput': ('00_data/core.html#tensorscalarsinput', 'tsfast/data/core.py'),
                                  'tsfast.data.core.TensorScalarsOutput': ('00_data/core.html#tensorscalarsoutput', 'tsfast/data/core.py'),
                                  'tsfast.data.core.TensorSequences': ('00_data/core.html#tensorsequences', 'tsfast/data/core.py'),
                                  'tsfast.data.core.TensorSequences.from_hdf': ( '00_data/core.html#tensorsequences.from_hdf',
                                                                                 'tsfast/data/core.py'),
                                  'tsfast.data.core.TensorSequences.show': ( '00_data/core.html#tensorsequences.show',
                                                                             'tsfast/data/core.py'),
                                  'tsfast.data.core.TensorSequencesInput': ( '00_data/core.html#tensorsequencesinput',
                                                                             'tsfast/data/core.py'),
                                  'tsfast.data.core.TensorSequencesOutput': ( '00_data/core.html#tensorsequencesoutput',
                                                                              'tsfast/data/core.py'),
                                  'tsfast.data.core.ValidClmContains': ('00_data/core.html#validclmcontains', 'tsfast/data/core.py'),
                                  'tsfast.data.core.ValidClmIs': ('00_data/core.html#validclmis', 'tsfast/data/core.py'),
                                  'tsfast.data.core.apply_df_tfms': ('00_data/core.html#apply_df_tfms', 'tsfast/data/core.py'),
                                  'tsfast.data.core.calc_shift_offsets': ('00_data/core.html#calc_shift_offsets', 'tsfast/data/core.py'),
                                  'tsfast.data.core.count_parameters': ('00_data/core.html#count_parameters', 'tsfast/data/core.py'),
                                  'tsfast.data.core.df_get_hdf_seq_len': ('00_data/core.html#df_get_hdf_seq_len', 'tsfast/data/core.py'),
                                  'tsfast.data.core.downsample_mean': ('00_data/core.html#downsample_mean', 'tsfast/data/core.py'),
                                  'tsfast.data.core.get_hdf_files': ('00_data/core.html#get_hdf_files', 'tsfast/data/core.py'),
                                  'tsfast.data.core.get_hdf_seq_len': ('00_data/core.html#get_hdf_seq_len', 'tsfast/data/core.py'),
                                  'tsfast.data.core.hdf2scalars': ('00_data/core.html#hdf2scalars', 'tsfast/data/core.py'),
                                  'tsfast.data.core.hdf_extract_sequence': ( '00_data/core.html#hdf_extract_sequence',
                                                                             'tsfast/data/core.py'),
                                  'tsfast.data.core.obj_in_lst': ('00_data/core.html#obj_in_lst', 'tsfast/data/core.py'),
                                  'tsfast.data.core.plot_seqs_multi_figures': ( '00_data/core.html#plot_seqs_multi_figures',
                                                                                'tsfast/data/core.py'),
                                  'tsfast.data.core.plot_seqs_single_figure': ( '00_data/core.html#plot_seqs_single_figure',
                                                                                'tsfast/data/core.py'),
                                  'tsfast.data.core.plot_sequence': ('00_data/core.html#plot_sequence', 'tsfast/data/core.py'),
                                  'tsfast.data.core.resample_interp': ('00_data/core.html#resample_interp', 'tsfast/data/core.py'),
                                  'tsfast.data.core.running_mean': ('00_data/core.html#running_mean', 'tsfast/data/core.py'),
                                  'tsfast.data.core.show_batch': ('00_data/core.html#show_batch', 'tsfast/data/core.py'),
                                  'tsfast.data.core.show_results': ('00_data/core.html#show_results', 'tsfast/data/core.py'),
                                  'tsfast.data.core.toTensorSequencesInput': ( '00_data/core.html#totensorsequencesinput',
                                                                               'tsfast/data/core.py'),
                                  'tsfast.data.core.toTensorSequencesOutput': ( '00_data/core.html#totensorsequencesoutput',
                                                                                'tsfast/data/core.py')},
            'tsfast.data.loader': { 'tsfast.data.loader.BatchLimit_Factory': ( '00_data/loader.html#batchlimit_factory',
                                                                               'tsfast/data/loader.py'),
                                    'tsfast.data.loader.TbpttDl': ('00_data/loader.html#tbpttdl', 'tsfast/data/loader.py'),
                                    'tsfast.data.loader.TbpttDl.__init__': ( '00_data/loader.html#tbpttdl.__init__',
                                                                             'tsfast/data/loader.py'),
                                    'tsfast.data.loader.TbpttDl.__iter__': ( '00_data/loader.html#tbpttdl.__iter__',
                                                                             'tsfast/data/loader.py'),
                                    'tsfast.data.loader.TbpttDl.__len__': ('00_data/loader.html#tbpttdl.__len__', 'tsfast/data/loader.py'),
                                    'tsfast.data.loader.TbpttDl._next_worker': ( '00_data/loader.html#tbpttdl._next_worker',
                                                                                 'tsfast/data/loader.py'),
                                    'tsfast.data.loader.TbpttDl._tbptt_generator': ( '00_data/loader.html#tbpttdl._tbptt_generator',
                                                                                     'tsfast/data/loader.py'),
                                    'tsfast.data.loader.TbpttDl.create_batches': ( '00_data/loader.html#tbpttdl.create_batches',
                                                                                   'tsfast/data/loader.py'),
                                    'tsfast.data.loader.TbpttDl.n_sub_seq': ( '00_data/loader.html#tbpttdl.n_sub_seq',
                                                                              'tsfast/data/loader.py'),
                                    'tsfast.data.loader.TbpttDl.sample': ('00_data/loader.html#tbpttdl.sample', 'tsfast/data/loader.py'),
                                    'tsfast.data.loader.TbpttResetCB': ('00_data/loader.html#tbpttresetcb', 'tsfast/data/loader.py'),
                                    'tsfast.data.loader.TbpttResetCB.after_fit': ( '00_data/loader.html#tbpttresetcb.after_fit',
                                                                                   'tsfast/data/loader.py'),
                                    'tsfast.data.loader.TbpttResetCB.before_batch': ( '00_data/loader.html#tbpttresetcb.before_batch',
                                                                                      'tsfast/data/loader.py'),
                                    'tsfast.data.loader.WeightedDL_Factory': ( '00_data/loader.html#weighteddl_factory',
                                                                               'tsfast/data/loader.py'),
                                    'tsfast.data.loader.reset_model_state': ( '00_data/loader.html#reset_model_state',
                                                                              'tsfast/data/loader.py'),
                                    'tsfast.data.loader.uniform_p_of_category': ( '00_data/loader.html#uniform_p_of_category',
                                                                                  'tsfast/data/loader.py'),
                                    'tsfast.data.loader.uniform_p_of_float': ( '00_data/loader.html#uniform_p_of_float',
                                                                               'tsfast/data/loader.py'),
                                    'tsfast.data.loader.uniform_p_of_float_with_gaps': ( '00_data/loader.html#uniform_p_of_float_with_gaps',
                                                                                         'tsfast/data/loader.py')},
            'tsfast.data.split': { 'tsfast.data.split.ApplyToDict': ('00_data/split.html#applytodict', 'tsfast/data/split.py'),
                                   'tsfast.data.split.ParentSplitter': ('00_data/split.html#parentsplitter', 'tsfast/data/split.py'),
                                   'tsfast.data.split.PercentageSplitter': ( '00_data/split.html#percentagesplitter',
                                                                             'tsfast/data/split.py'),
                                   'tsfast.data.split._parent_idxs': ('00_data/split.html#_parent_idxs', 'tsfast/data/split.py')},
            'tsfast.data.transforms': { 'tsfast.data.transforms.SeqBiasInjection': ( '00_data/transforms.html#seqbiasinjection',
                                                                                     'tsfast/data/transforms.py'),
                                        'tsfast.data.transforms.SeqBiasInjection.__init__': ( '00_data/transforms.html#seqbiasinjection.__init__',
                                                                                              'tsfast/data/transforms.py'),
                                        'tsfast.data.transforms.SeqBiasInjection.encodes': ( '00_data/transforms.html#seqbiasinjection.encodes',
                                                                                             'tsfast/data/transforms.py'),
                                        'tsfast.data.transforms.SeqNoiseInjection': ( '00_data/transforms.html#seqnoiseinjection',
                                                                                      'tsfast/data/transforms.py'),
                                        'tsfast.data.transforms.SeqNoiseInjection.__init__': ( '00_data/transforms.html#seqnoiseinjection.__init__',
                                                                                               'tsfast/data/transforms.py'),
                                        'tsfast.data.transforms.SeqNoiseInjection.encodes': ( '00_data/transforms.html#seqnoiseinjection.encodes',
                                                                                              'tsfast/data/transforms.py'),
                                        'tsfast.data.transforms.SeqNoiseInjection_Grouped': ( '00_data/transforms.html#seqnoiseinjection_grouped',
                                                                                              'tsfast/data/transforms.py'),
                                        'tsfast.data.transforms.SeqNoiseInjection_Grouped.__init__': ( '00_data/transforms.html#seqnoiseinjection_grouped.__init__',
                                                                                                       'tsfast/data/transforms.py'),
                                        'tsfast.data.transforms.SeqNoiseInjection_Grouped.encodes': ( '00_data/transforms.html#seqnoiseinjection_grouped.encodes',
                                                                                                      'tsfast/data/transforms.py'),
                                        'tsfast.data.transforms.SeqNoiseInjection_Varying': ( '00_data/transforms.html#seqnoiseinjection_varying',
                                                                                              'tsfast/data/transforms.py'),
                                        'tsfast.data.transforms.SeqNoiseInjection_Varying.__init__': ( '00_data/transforms.html#seqnoiseinjection_varying.__init__',
                                                                                                       'tsfast/data/transforms.py'),
                                        'tsfast.data.transforms.SeqNoiseInjection_Varying.encodes': ( '00_data/transforms.html#seqnoiseinjection_varying.encodes',
                                                                                                      'tsfast/data/transforms.py'),
                                        'tsfast.data.transforms.SeqSlice': ( '00_data/transforms.html#seqslice',
                                                                             'tsfast/data/transforms.py'),
                                        'tsfast.data.transforms.SeqSlice.__init__': ( '00_data/transforms.html#seqslice.__init__',
                                                                                      'tsfast/data/transforms.py'),
                                        'tsfast.data.transforms.SeqSlice.encodes': ( '00_data/transforms.html#seqslice.encodes',
                                                                                     'tsfast/data/transforms.py'),
                                        'tsfast.data.transforms.decodes': ('00_data/transforms.html#decodes', 'tsfast/data/transforms.py'),
                                        'tsfast.data.transforms.encodes': ('00_data/transforms.html#encodes', 'tsfast/data/transforms.py')},
            'tsfast.datasets.benchmark': { 'tsfast.datasets.benchmark.create_dls_from_spec': ( '01_datasets/benchmark.html#create_dls_from_spec',
                                                                                               'tsfast/datasets/benchmark.py')},
            'tsfast.datasets.core': { 'tsfast.datasets.core.clean_default_dataset_path': ( '01_datasets/core.html#clean_default_dataset_path',
                                                                                           'tsfast/datasets/core.py'),
                                      'tsfast.datasets.core.create_dls': ('01_datasets/core.html#create_dls', 'tsfast/datasets/core.py'),
                                      'tsfast.datasets.core.create_dls_downl': ( '01_datasets/core.html#create_dls_downl',
                                                                                 'tsfast/datasets/core.py'),
                                      'tsfast.datasets.core.dict_file_load': ( '01_datasets/core.html#dict_file_load',
                                                                               'tsfast/datasets/core.py'),
                                      'tsfast.datasets.core.dict_file_save': ( '01_datasets/core.html#dict_file_save',
                                                                               'tsfast/datasets/core.py'),
                                      'tsfast.datasets.core.extract_mean_std_from_dataset': ( '01_datasets/core.html#extract_mean_std_from_dataset',
                                                                                              'tsfast/datasets/core.py'),
                                      'tsfast.datasets.core.extract_mean_std_from_dls': ( '01_datasets/core.html#extract_mean_std_from_dls',
                                                                                          'tsfast/datasets/core.py'),
                                      'tsfast.datasets.core.extract_mean_std_from_hdffiles': ( '01_datasets/core.html#extract_mean_std_from_hdffiles',
                                                                                               'tsfast/datasets/core.py'),
                                      'tsfast.datasets.core.get_dataset_path': ( '01_datasets/core.html#get_dataset_path',
                                                                                 'tsfast/datasets/core.py'),
                                      'tsfast.datasets.core.get_default_dataset_path': ( '01_datasets/core.html#get_default_dataset_path',
                                                                                         'tsfast/datasets/core.py'),
                                      'tsfast.datasets.core.is_dataset_directory': ( '01_datasets/core.html#is_dataset_directory',
                                                                                     'tsfast/datasets/core.py')},
            'tsfast.inference.core': { 'tsfast.inference.core.InferenceWrapper': ( '05_inference/core.html#inferencewrapper',
                                                                                   'tsfast/inference/core.py'),
                                       'tsfast.inference.core.InferenceWrapper.__call__': ( '05_inference/core.html#inferencewrapper.__call__',
                                                                                            'tsfast/inference/core.py'),
                                       'tsfast.inference.core.InferenceWrapper.__init__': ( '05_inference/core.html#inferencewrapper.__init__',
                                                                                            'tsfast/inference/core.py'),
                                       'tsfast.inference.core.InferenceWrapper._adjust_seq_len': ( '05_inference/core.html#inferencewrapper._adjust_seq_len',
                                                                                                   'tsfast/inference/core.py'),
                                       'tsfast.inference.core.InferenceWrapper._prepare_tensor': ( '05_inference/core.html#inferencewrapper._prepare_tensor',
                                                                                                   'tsfast/inference/core.py'),
                                       'tsfast.inference.core.InferenceWrapper.inference': ( '05_inference/core.html#inferencewrapper.inference',
                                                                                             'tsfast/inference/core.py')},
            'tsfast.learner.callbacks': { 'tsfast.learner.callbacks.ARInitCB': ( '02_learner/callbacks.html#arinitcb',
                                                                                 'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.ARInitCB.before_batch': ( '02_learner/callbacks.html#arinitcb.before_batch',
                                                                                              'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.BatchLossFilter': ( '02_learner/callbacks.html#batchlossfilter',
                                                                                        'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.BatchLossFilter.__init__': ( '02_learner/callbacks.html#batchlossfilter.__init__',
                                                                                                 'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.BatchLossFilter.after_pred': ( '02_learner/callbacks.html#batchlossfilter.after_pred',
                                                                                                   'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.CB_AddLoss': ( '02_learner/callbacks.html#cb_addloss',
                                                                                   'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.CB_AddLoss.__init__': ( '02_learner/callbacks.html#cb_addloss.__init__',
                                                                                            'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.CB_AddLoss.after_loss': ( '02_learner/callbacks.html#cb_addloss.after_loss',
                                                                                              'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.CB_PlotGradient': ( '02_learner/callbacks.html#cb_plotgradient',
                                                                                        'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.CB_PlotGradient.__init__': ( '02_learner/callbacks.html#cb_plotgradient.__init__',
                                                                                                 'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.CB_PlotGradient.after_backward': ( '02_learner/callbacks.html#cb_plotgradient.after_backward',
                                                                                                       'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.CB_PlotGradient.begin_fit': ( '02_learner/callbacks.html#cb_plotgradient.begin_fit',
                                                                                                  'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.CB_TruncateSequence': ( '02_learner/callbacks.html#cb_truncatesequence',
                                                                                            'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.CB_TruncateSequence.__init__': ( '02_learner/callbacks.html#cb_truncatesequence.__init__',
                                                                                                     'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.CB_TruncateSequence.before_batch': ( '02_learner/callbacks.html#cb_truncatesequence.before_batch',
                                                                                                         'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.CancelNaNCallback': ( '02_learner/callbacks.html#cancelnancallback',
                                                                                          'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.CancelNaNCallback.after_loss': ( '02_learner/callbacks.html#cancelnancallback.after_loss',
                                                                                                     'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.GradientBatchFiltering': ( '02_learner/callbacks.html#gradientbatchfiltering',
                                                                                               'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.GradientBatchFiltering.__init__': ( '02_learner/callbacks.html#gradientbatchfiltering.__init__',
                                                                                                        'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.GradientBatchFiltering.before_step': ( '02_learner/callbacks.html#gradientbatchfiltering.before_step',
                                                                                                           'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.GradientClipping': ( '02_learner/callbacks.html#gradientclipping',
                                                                                         'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.GradientClipping.__init__': ( '02_learner/callbacks.html#gradientclipping.__init__',
                                                                                                  'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.GradientClipping.after_backward': ( '02_learner/callbacks.html#gradientclipping.after_backward',
                                                                                                        'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.GradientNormPrint': ( '02_learner/callbacks.html#gradientnormprint',
                                                                                          'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.GradientNormPrint.before_step': ( '02_learner/callbacks.html#gradientnormprint.before_step',
                                                                                                      'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.SkipFirstNCallback': ( '02_learner/callbacks.html#skipfirstncallback',
                                                                                           'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.SkipFirstNCallback.__init__': ( '02_learner/callbacks.html#skipfirstncallback.__init__',
                                                                                                    'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.SkipFirstNCallback.after_pred': ( '02_learner/callbacks.html#skipfirstncallback.after_pred',
                                                                                                      'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.SkipNaNCallback': ( '02_learner/callbacks.html#skipnancallback',
                                                                                        'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.SkipNaNCallback.after_loss': ( '02_learner/callbacks.html#skipnancallback.after_loss',
                                                                                                   'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.TimeSeriesRegularizer': ( '02_learner/callbacks.html#timeseriesregularizer',
                                                                                              'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.TimeSeriesRegularizer.__init__': ( '02_learner/callbacks.html#timeseriesregularizer.__init__',
                                                                                                       'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.TimeSeriesRegularizer.after_loss': ( '02_learner/callbacks.html#timeseriesregularizer.after_loss',
                                                                                                         'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.TimeSeriesRegularizer.hook': ( '02_learner/callbacks.html#timeseriesregularizer.hook',
                                                                                                   'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.VarySeqLen': ( '02_learner/callbacks.html#varyseqlen',
                                                                                   'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.VarySeqLen.__init__': ( '02_learner/callbacks.html#varyseqlen.__init__',
                                                                                            'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.VarySeqLen.before_batch': ( '02_learner/callbacks.html#varyseqlen.before_batch',
                                                                                                'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.WeightClipping': ( '02_learner/callbacks.html#weightclipping',
                                                                                       'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.WeightClipping.__init__': ( '02_learner/callbacks.html#weightclipping.__init__',
                                                                                                'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.WeightClipping.after_batch': ( '02_learner/callbacks.html#weightclipping.after_batch',
                                                                                                   'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.plot_grad_flow': ( '02_learner/callbacks.html#plot_grad_flow',
                                                                                       'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.sched_lin_p': ( '02_learner/callbacks.html#sched_lin_p',
                                                                                    'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.sched_ramp': ( '02_learner/callbacks.html#sched_ramp',
                                                                                   'tsfast/learner/callbacks.py')},
            'tsfast.learner.learner': { 'tsfast.learner.learner.AR_RNNLearner': ( '02_learner/learner.html#ar_rnnlearner',
                                                                                  'tsfast/learner/learner.py'),
                                        'tsfast.learner.learner.AR_TCNLearner': ( '02_learner/learner.html#ar_tcnlearner',
                                                                                  'tsfast/learner/learner.py'),
                                        'tsfast.learner.learner.CRNNLearner': ( '02_learner/learner.html#crnnlearner',
                                                                                'tsfast/learner/learner.py'),
                                        'tsfast.learner.learner.RNNLearner': ( '02_learner/learner.html#rnnlearner',
                                                                               'tsfast/learner/learner.py'),
                                        'tsfast.learner.learner.TCNLearner': ( '02_learner/learner.html#tcnlearner',
                                                                               'tsfast/learner/learner.py'),
                                        'tsfast.learner.learner.get_inp_out_size': ( '02_learner/learner.html#get_inp_out_size',
                                                                                     'tsfast/learner/learner.py')},
            'tsfast.learner.losses': { 'tsfast.learner.losses.CutLoss': ('02_learner/losses.html#cutloss', 'tsfast/learner/losses.py'),
                                       'tsfast.learner.losses.RandSeqLenLoss': ( '02_learner/losses.html#randseqlenloss',
                                                                                 'tsfast/learner/losses.py'),
                                       'tsfast.learner.losses.SkipNLoss': ('02_learner/losses.html#skipnloss', 'tsfast/learner/losses.py'),
                                       'tsfast.learner.losses.cos_sim_loss': ( '02_learner/losses.html#cos_sim_loss',
                                                                               'tsfast/learner/losses.py'),
                                       'tsfast.learner.losses.cos_sim_loss_pow': ( '02_learner/losses.html#cos_sim_loss_pow',
                                                                                   'tsfast/learner/losses.py'),
                                       'tsfast.learner.losses.float64_func': ( '02_learner/losses.html#float64_func',
                                                                               'tsfast/learner/losses.py'),
                                       'tsfast.learner.losses.fun_rmse': ('02_learner/losses.html#fun_rmse', 'tsfast/learner/losses.py'),
                                       'tsfast.learner.losses.ignore_nan': ( '02_learner/losses.html#ignore_nan',
                                                                             'tsfast/learner/losses.py'),
                                       'tsfast.learner.losses.mean_vaf': ('02_learner/losses.html#mean_vaf', 'tsfast/learner/losses.py'),
                                       'tsfast.learner.losses.nrmse': ('02_learner/losses.html#nrmse', 'tsfast/learner/losses.py'),
                                       'tsfast.learner.losses.nrmse_std': ('02_learner/losses.html#nrmse_std', 'tsfast/learner/losses.py'),
                                       'tsfast.learner.losses.weighted_mae': ( '02_learner/losses.html#weighted_mae',
                                                                               'tsfast/learner/losses.py')},
            'tsfast.models.cnn': { 'tsfast.models.cnn.CConv1D': ('03_models/cnn.html#cconv1d', 'tsfast/models/cnn.py'),
                                   'tsfast.models.cnn.CNN': ('03_models/cnn.html#cnn', 'tsfast/models/cnn.py'),
                                   'tsfast.models.cnn.CNN.__init__': ('03_models/cnn.html#cnn.__init__', 'tsfast/models/cnn.py'),
                                   'tsfast.models.cnn.CNN.forward': ('03_models/cnn.html#cnn.forward', 'tsfast/models/cnn.py'),
                                   'tsfast.models.cnn.CRNN': ('03_models/cnn.html#crnn', 'tsfast/models/cnn.py'),
                                   'tsfast.models.cnn.CRNN.__init__': ('03_models/cnn.html#crnn.__init__', 'tsfast/models/cnn.py'),
                                   'tsfast.models.cnn.CRNN.forward': ('03_models/cnn.html#crnn.forward', 'tsfast/models/cnn.py'),
                                   'tsfast.models.cnn.CausalConv1d': ('03_models/cnn.html#causalconv1d', 'tsfast/models/cnn.py'),
                                   'tsfast.models.cnn.CausalConv1d.__init__': ( '03_models/cnn.html#causalconv1d.__init__',
                                                                                'tsfast/models/cnn.py'),
                                   'tsfast.models.cnn.CausalConv1d.forward': ( '03_models/cnn.html#causalconv1d.forward',
                                                                               'tsfast/models/cnn.py'),
                                   'tsfast.models.cnn.CausalConv1d.reset_state': ( '03_models/cnn.html#causalconv1d.reset_state',
                                                                                   'tsfast/models/cnn.py'),
                                   'tsfast.models.cnn.Conv1D': ('03_models/cnn.html#conv1d', 'tsfast/models/cnn.py'),
                                   'tsfast.models.cnn.SeperateCRNN': ('03_models/cnn.html#seperatecrnn', 'tsfast/models/cnn.py'),
                                   'tsfast.models.cnn.SeperateCRNN.__init__': ( '03_models/cnn.html#seperatecrnn.__init__',
                                                                                'tsfast/models/cnn.py'),
                                   'tsfast.models.cnn.SeperateCRNN.forward': ( '03_models/cnn.html#seperatecrnn.forward',
                                                                               'tsfast/models/cnn.py'),
                                   'tsfast.models.cnn.SeperateTCN': ('03_models/cnn.html#seperatetcn', 'tsfast/models/cnn.py'),
                                   'tsfast.models.cnn.SeperateTCN.__init__': ( '03_models/cnn.html#seperatetcn.__init__',
                                                                               'tsfast/models/cnn.py'),
                                   'tsfast.models.cnn.SeperateTCN.forward': ( '03_models/cnn.html#seperatetcn.forward',
                                                                              'tsfast/models/cnn.py'),
                                   'tsfast.models.cnn.SeperateTCN.reset_state': ( '03_models/cnn.html#seperatetcn.reset_state',
                                                                                  'tsfast/models/cnn.py'),
                                   'tsfast.models.cnn.TCN': ('03_models/cnn.html#tcn', 'tsfast/models/cnn.py'),
                                   'tsfast.models.cnn.TCN.__init__': ('03_models/cnn.html#tcn.__init__', 'tsfast/models/cnn.py'),
                                   'tsfast.models.cnn.TCN.forward': ('03_models/cnn.html#tcn.forward', 'tsfast/models/cnn.py'),
                                   'tsfast.models.cnn.TCN_Block': ('03_models/cnn.html#tcn_block', 'tsfast/models/cnn.py'),
                                   'tsfast.models.cnn.TCN_Block.__init__': ( '03_models/cnn.html#tcn_block.__init__',
                                                                             'tsfast/models/cnn.py'),
                                   'tsfast.models.cnn.TCN_Block.forward': ('03_models/cnn.html#tcn_block.forward', 'tsfast/models/cnn.py')},
            'tsfast.models.layers': { 'tsfast.models.layers.AR_Model': ('03_models/layers.html#ar_model', 'tsfast/models/layers.py'),
                                      'tsfast.models.layers.AR_Model.__init__': ( '03_models/layers.html#ar_model.__init__',
                                                                                  'tsfast/models/layers.py'),
                                      'tsfast.models.layers.AR_Model.forward': ( '03_models/layers.html#ar_model.forward',
                                                                                 'tsfast/models/layers.py'),
                                      'tsfast.models.layers.AR_Model.init_normalize': ( '03_models/layers.html#ar_model.init_normalize',
                                                                                        'tsfast/models/layers.py'),
                                      'tsfast.models.layers.AR_Model.init_normalize_values': ( '03_models/layers.html#ar_model.init_normalize_values',
                                                                                               'tsfast/models/layers.py'),
                                      'tsfast.models.layers.AR_Model.reset_state': ( '03_models/layers.html#ar_model.reset_state',
                                                                                     'tsfast/models/layers.py'),
                                      'tsfast.models.layers.BatchNorm_1D_Stateful': ( '03_models/layers.html#batchnorm_1d_stateful',
                                                                                      'tsfast/models/layers.py'),
                                      'tsfast.models.layers.BatchNorm_1D_Stateful.__init__': ( '03_models/layers.html#batchnorm_1d_stateful.__init__',
                                                                                               'tsfast/models/layers.py'),
                                      'tsfast.models.layers.BatchNorm_1D_Stateful.forward': ( '03_models/layers.html#batchnorm_1d_stateful.forward',
                                                                                              'tsfast/models/layers.py'),
                                      'tsfast.models.layers.BatchNorm_1D_Stateful.reset_parameters': ( '03_models/layers.html#batchnorm_1d_stateful.reset_parameters',
                                                                                                       'tsfast/models/layers.py'),
                                      'tsfast.models.layers.BatchNorm_1D_Stateful.reset_state': ( '03_models/layers.html#batchnorm_1d_stateful.reset_state',
                                                                                                  'tsfast/models/layers.py'),
                                      'tsfast.models.layers.NormalizedModel': ( '03_models/layers.html#normalizedmodel',
                                                                                'tsfast/models/layers.py'),
                                      'tsfast.models.layers.NormalizedModel.__init__': ( '03_models/layers.html#normalizedmodel.__init__',
                                                                                         'tsfast/models/layers.py'),
                                      'tsfast.models.layers.NormalizedModel.forward': ( '03_models/layers.html#normalizedmodel.forward',
                                                                                        'tsfast/models/layers.py'),
                                      'tsfast.models.layers.NormalizedModel.from_dls': ( '03_models/layers.html#normalizedmodel.from_dls',
                                                                                         'tsfast/models/layers.py'),
                                      'tsfast.models.layers.Normalizer1D': ( '03_models/layers.html#normalizer1d',
                                                                             'tsfast/models/layers.py'),
                                      'tsfast.models.layers.Normalizer1D.__init__': ( '03_models/layers.html#normalizer1d.__init__',
                                                                                      'tsfast/models/layers.py'),
                                      'tsfast.models.layers.Normalizer1D.normalize': ( '03_models/layers.html#normalizer1d.normalize',
                                                                                       'tsfast/models/layers.py'),
                                      'tsfast.models.layers.Normalizer1D.unnormalize': ( '03_models/layers.html#normalizer1d.unnormalize',
                                                                                         'tsfast/models/layers.py'),
                                      'tsfast.models.layers.SeqLinear': ('03_models/layers.html#seqlinear', 'tsfast/models/layers.py'),
                                      'tsfast.models.layers.SeqLinear.__init__': ( '03_models/layers.html#seqlinear.__init__',
                                                                                   'tsfast/models/layers.py'),
                                      'tsfast.models.layers.SeqLinear.forward': ( '03_models/layers.html#seqlinear.forward',
                                                                                  'tsfast/models/layers.py')},
            'tsfast.models.rnn': { 'tsfast.models.rnn.DenseBlock_RNN': ('03_models/rnn.html#denseblock_rnn', 'tsfast/models/rnn.py'),
                                   'tsfast.models.rnn.DenseBlock_RNN.__init__': ( '03_models/rnn.html#denseblock_rnn.__init__',
                                                                                  'tsfast/models/rnn.py'),
                                   'tsfast.models.rnn.DenseLayer_RNN': ('03_models/rnn.html#denselayer_rnn', 'tsfast/models/rnn.py'),
                                   'tsfast.models.rnn.DenseLayer_RNN.__init__': ( '03_models/rnn.html#denselayer_rnn.__init__',
                                                                                  'tsfast/models/rnn.py'),
                                   'tsfast.models.rnn.DenseLayer_RNN.forward': ( '03_models/rnn.html#denselayer_rnn.forward',
                                                                                 'tsfast/models/rnn.py'),
                                   'tsfast.models.rnn.DenseNet_RNN': ('03_models/rnn.html#densenet_rnn', 'tsfast/models/rnn.py'),
                                   'tsfast.models.rnn.DenseNet_RNN.__init__': ( '03_models/rnn.html#densenet_rnn.__init__',
                                                                                'tsfast/models/rnn.py'),
                                   'tsfast.models.rnn.RNN': ('03_models/rnn.html#rnn', 'tsfast/models/rnn.py'),
                                   'tsfast.models.rnn.RNN.__init__': ('03_models/rnn.html#rnn.__init__', 'tsfast/models/rnn.py'),
                                   'tsfast.models.rnn.RNN._get_hidden': ('03_models/rnn.html#rnn._get_hidden', 'tsfast/models/rnn.py'),
                                   'tsfast.models.rnn.RNN._one_rnn': ('03_models/rnn.html#rnn._one_rnn', 'tsfast/models/rnn.py'),
                                   'tsfast.models.rnn.RNN.forward': ('03_models/rnn.html#rnn.forward', 'tsfast/models/rnn.py'),
                                   'tsfast.models.rnn.RNN.reset_state': ('03_models/rnn.html#rnn.reset_state', 'tsfast/models/rnn.py'),
                                   'tsfast.models.rnn.ResidualBlock_RNN': ('03_models/rnn.html#residualblock_rnn', 'tsfast/models/rnn.py'),
                                   'tsfast.models.rnn.ResidualBlock_RNN.__init__': ( '03_models/rnn.html#residualblock_rnn.__init__',
                                                                                     'tsfast/models/rnn.py'),
                                   'tsfast.models.rnn.ResidualBlock_RNN.forward': ( '03_models/rnn.html#residualblock_rnn.forward',
                                                                                    'tsfast/models/rnn.py'),
                                   'tsfast.models.rnn.SeperateRNN': ('03_models/rnn.html#seperaternn', 'tsfast/models/rnn.py'),
                                   'tsfast.models.rnn.SeperateRNN.__init__': ( '03_models/rnn.html#seperaternn.__init__',
                                                                               'tsfast/models/rnn.py'),
                                   'tsfast.models.rnn.SeperateRNN.forward': ( '03_models/rnn.html#seperaternn.forward',
                                                                              'tsfast/models/rnn.py'),
                                   'tsfast.models.rnn.Sequential_RNN': ('03_models/rnn.html#sequential_rnn', 'tsfast/models/rnn.py'),
                                   'tsfast.models.rnn.Sequential_RNN.forward': ( '03_models/rnn.html#sequential_rnn.forward',
                                                                                 'tsfast/models/rnn.py'),
                                   'tsfast.models.rnn.SimpleRNN': ('03_models/rnn.html#simplernn', 'tsfast/models/rnn.py'),
                                   'tsfast.models.rnn.SimpleRNN.__init__': ( '03_models/rnn.html#simplernn.__init__',
                                                                             'tsfast/models/rnn.py'),
                                   'tsfast.models.rnn.SimpleRNN.forward': ('03_models/rnn.html#simplernn.forward', 'tsfast/models/rnn.py'),
                                   'tsfast.models.rnn.SimpleResidualRNN': ('03_models/rnn.html#simpleresidualrnn', 'tsfast/models/rnn.py'),
                                   'tsfast.models.rnn.SimpleResidualRNN.__init__': ( '03_models/rnn.html#simpleresidualrnn.__init__',
                                                                                     'tsfast/models/rnn.py')},
            'tsfast.prediction.ar': { 'tsfast.prediction.ar.ARProg': ('04_prediction/ar.html#arprog', 'tsfast/prediction/ar.py'),
                                      'tsfast.prediction.ar.ARProg.__init__': ( '04_prediction/ar.html#arprog.__init__',
                                                                                'tsfast/prediction/ar.py'),
                                      'tsfast.prediction.ar.ARProg.forward': ( '04_prediction/ar.html#arprog.forward',
                                                                               'tsfast/prediction/ar.py')},
            'tsfast.prediction.core': { 'tsfast.prediction.core.PredictionCallback': ( '04_prediction/core.html#predictioncallback',
                                                                                       'tsfast/prediction/core.py'),
                                        'tsfast.prediction.core.PredictionCallback.__init__': ( '04_prediction/core.html#predictioncallback.__init__',
                                                                                                'tsfast/prediction/core.py'),
                                        'tsfast.prediction.core.PredictionCallback.before_batch': ( '04_prediction/core.html#predictioncallback.before_batch',
                                                                                                    'tsfast/prediction/core.py'),
                                        'tsfast.prediction.core.PredictionCallback.init_normalize': ( '04_prediction/core.html#predictioncallback.init_normalize',
                                                                                                      'tsfast/prediction/core.py'),
                                        'tsfast.prediction.core.PredictionCallback.init_normalize_values': ( '04_prediction/core.html#predictioncallback.init_normalize_values',
                                                                                                             'tsfast/prediction/core.py')},
            'tsfast.prediction.fransys': { 'tsfast.prediction.fransys.ARProg_Init': ( '04_prediction/fransys.html#arprog_init',
                                                                                      'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.ARProg_Init.__init__': ( '04_prediction/fransys.html#arprog_init.__init__',
                                                                                               'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.ARProg_Init.forward': ( '04_prediction/fransys.html#arprog_init.forward',
                                                                                              'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.DiagLSTM': ( '04_prediction/fransys.html#diaglstm',
                                                                                   'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.DiagLSTM.__init__': ( '04_prediction/fransys.html#diaglstm.__init__',
                                                                                            'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.DiagLSTM._get_hidden': ( '04_prediction/fransys.html#diaglstm._get_hidden',
                                                                                               'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.DiagLSTM.forward': ( '04_prediction/fransys.html#diaglstm.forward',
                                                                                           'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.DiagLSTM.output_to_hidden': ( '04_prediction/fransys.html#diaglstm.output_to_hidden',
                                                                                                    'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.Diag_RNN': ( '04_prediction/fransys.html#diag_rnn',
                                                                                   'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.Diag_RNN.__init__': ( '04_prediction/fransys.html#diag_rnn.__init__',
                                                                                            'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.Diag_RNN._get_hidden': ( '04_prediction/fransys.html#diag_rnn._get_hidden',
                                                                                               'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.Diag_RNN.forward': ( '04_prediction/fransys.html#diag_rnn.forward',
                                                                                           'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.Diag_RNN.output_to_hidden': ( '04_prediction/fransys.html#diag_rnn.output_to_hidden',
                                                                                                    'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.Diag_RNN_raw': ( '04_prediction/fransys.html#diag_rnn_raw',
                                                                                       'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.Diag_RNN_raw.__init__': ( '04_prediction/fransys.html#diag_rnn_raw.__init__',
                                                                                                'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.Diag_RNN_raw.forward': ( '04_prediction/fransys.html#diag_rnn_raw.forward',
                                                                                               'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.Diag_RNN_raw.output_to_hidden': ( '04_prediction/fransys.html#diag_rnn_raw.output_to_hidden',
                                                                                                        'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.Diag_TCN': ( '04_prediction/fransys.html#diag_tcn',
                                                                                   'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.Diag_TCN.__init__': ( '04_prediction/fransys.html#diag_tcn.__init__',
                                                                                            'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.Diag_TCN.forward': ( '04_prediction/fransys.html#diag_tcn.forward',
                                                                                           'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.Diag_TCN.output_to_hidden': ( '04_prediction/fransys.html#diag_tcn.output_to_hidden',
                                                                                                    'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.FranSys': ( '04_prediction/fransys.html#fransys',
                                                                                  'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.FranSys.__init__': ( '04_prediction/fransys.html#fransys.__init__',
                                                                                           'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.FranSys.forward': ( '04_prediction/fransys.html#fransys.forward',
                                                                                          'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.FranSysCallback': ( '04_prediction/fransys.html#fransyscallback',
                                                                                          'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.FranSysCallback.__init__': ( '04_prediction/fransys.html#fransyscallback.__init__',
                                                                                                   'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.FranSysCallback.after_loss': ( '04_prediction/fransys.html#fransyscallback.after_loss',
                                                                                                     'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.FranSysCallback.before_batch': ( '04_prediction/fransys.html#fransyscallback.before_batch',
                                                                                                       'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.FranSysCallback.clear': ( '04_prediction/fransys.html#fransyscallback.clear',
                                                                                                'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.FranSysCallback.hook': ( '04_prediction/fransys.html#fransyscallback.hook',
                                                                                               'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.FranSysCallback_variable_init': ( '04_prediction/fransys.html#fransyscallback_variable_init',
                                                                                                        'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.FranSysCallback_variable_init.__init__': ( '04_prediction/fransys.html#fransyscallback_variable_init.__init__',
                                                                                                                 'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.FranSysCallback_variable_init.before_batch': ( '04_prediction/fransys.html#fransyscallback_variable_init.before_batch',
                                                                                                                     'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.FranSysLearner': ( '04_prediction/fransys.html#fransyslearner',
                                                                                         'tsfast/prediction/fransys.py')},
            'tsfast.quaternions': { 'tsfast.quaternions.HDF2Inclination': ('quaternions.html#hdf2inclination', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.HDF2Inclination._hdf_extract_sequence': ( 'quaternions.html#hdf2inclination._hdf_extract_sequence',
                                                                                                  'tsfast/quaternions.py'),
                                    'tsfast.quaternions.HDF2Quaternion': ('quaternions.html#hdf2quaternion', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.HDF2Quaternion._hdf_extract_sequence': ( 'quaternions.html#hdf2quaternion._hdf_extract_sequence',
                                                                                                 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.InclinationBlock': ('quaternions.html#inclinationblock', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.InclinationBlock.__init__': ( 'quaternions.html#inclinationblock.__init__',
                                                                                      'tsfast/quaternions.py'),
                                    'tsfast.quaternions.InclinationBlock.from_hdf': ( 'quaternions.html#inclinationblock.from_hdf',
                                                                                      'tsfast/quaternions.py'),
                                    'tsfast.quaternions.QuaternionAugmentation': ( 'quaternions.html#quaternionaugmentation',
                                                                                   'tsfast/quaternions.py'),
                                    'tsfast.quaternions.QuaternionAugmentation.__call__': ( 'quaternions.html#quaternionaugmentation.__call__',
                                                                                            'tsfast/quaternions.py'),
                                    'tsfast.quaternions.QuaternionAugmentation.__init__': ( 'quaternions.html#quaternionaugmentation.__init__',
                                                                                            'tsfast/quaternions.py'),
                                    'tsfast.quaternions.QuaternionAugmentation.encodes': ( 'quaternions.html#quaternionaugmentation.encodes',
                                                                                           'tsfast/quaternions.py'),
                                    'tsfast.quaternions.QuaternionBlock': ('quaternions.html#quaternionblock', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.QuaternionBlock.__init__': ( 'quaternions.html#quaternionblock.__init__',
                                                                                     'tsfast/quaternions.py'),
                                    'tsfast.quaternions.QuaternionBlock.from_hdf': ( 'quaternions.html#quaternionblock.from_hdf',
                                                                                     'tsfast/quaternions.py'),
                                    'tsfast.quaternions.QuaternionRegularizer': ( 'quaternions.html#quaternionregularizer',
                                                                                  'tsfast/quaternions.py'),
                                    'tsfast.quaternions.QuaternionRegularizer.__init__': ( 'quaternions.html#quaternionregularizer.__init__',
                                                                                           'tsfast/quaternions.py'),
                                    'tsfast.quaternions.QuaternionRegularizer.after_loss': ( 'quaternions.html#quaternionregularizer.after_loss',
                                                                                             'tsfast/quaternions.py'),
                                    'tsfast.quaternions.QuaternionRegularizer.hook': ( 'quaternions.html#quaternionregularizer.hook',
                                                                                       'tsfast/quaternions.py'),
                                    'tsfast.quaternions.Quaternion_ResamplingModel': ( 'quaternions.html#quaternion_resamplingmodel',
                                                                                       'tsfast/quaternions.py'),
                                    'tsfast.quaternions.Quaternion_ResamplingModel.__init__': ( 'quaternions.html#quaternion_resamplingmodel.__init__',
                                                                                                'tsfast/quaternions.py'),
                                    'tsfast.quaternions.Quaternion_ResamplingModel.forward': ( 'quaternions.html#quaternion_resamplingmodel.forward',
                                                                                               'tsfast/quaternions.py'),
                                    'tsfast.quaternions.TensorInclination': ('quaternions.html#tensorinclination', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.TensorQuaternionAngle': ( 'quaternions.html#tensorquaternionangle',
                                                                                  'tsfast/quaternions.py'),
                                    'tsfast.quaternions.TensorQuaternionInclination': ( 'quaternions.html#tensorquaternioninclination',
                                                                                        'tsfast/quaternions.py'),
                                    'tsfast.quaternions.abs_inclination': ('quaternions.html#abs_inclination', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.abs_rel_angle': ('quaternions.html#abs_rel_angle', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.angle_loss': ('quaternions.html#angle_loss', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.angle_loss_opt': ('quaternions.html#angle_loss_opt', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.augmentation_groups': ( 'quaternions.html#augmentation_groups',
                                                                                'tsfast/quaternions.py'),
                                    'tsfast.quaternions.conjQuat': ('quaternions.html#conjquat', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.deg_rmse': ('quaternions.html#deg_rmse', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.diffQuat': ('quaternions.html#diffquat', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.inclinationAngle': ('quaternions.html#inclinationangle', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.inclinationAngleAbs': ( 'quaternions.html#inclinationangleabs',
                                                                                'tsfast/quaternions.py'),
                                    'tsfast.quaternions.inclination_loss': ('quaternions.html#inclination_loss', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.inclination_loss_abs': ( 'quaternions.html#inclination_loss_abs',
                                                                                 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.inclination_loss_smooth': ( 'quaternions.html#inclination_loss_smooth',
                                                                                    'tsfast/quaternions.py'),
                                    'tsfast.quaternions.inclination_loss_squared': ( 'quaternions.html#inclination_loss_squared',
                                                                                     'tsfast/quaternions.py'),
                                    'tsfast.quaternions.mean_inclination_deg': ( 'quaternions.html#mean_inclination_deg',
                                                                                 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.mean_rel_angle_deg': ( 'quaternions.html#mean_rel_angle_deg',
                                                                               'tsfast/quaternions.py'),
                                    'tsfast.quaternions.ms_inclination': ('quaternions.html#ms_inclination', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.ms_rel_angle': ('quaternions.html#ms_rel_angle', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.multiplyQuat': ('quaternions.html#multiplyquat', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.multiplyQuat_np': ('quaternions.html#multiplyquat_np', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.norm_quaternion': ('quaternions.html#norm_quaternion', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.pitchAngle': ('quaternions.html#pitchangle', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.plot_quaternion_inclination': ( 'quaternions.html#plot_quaternion_inclination',
                                                                                        'tsfast/quaternions.py'),
                                    'tsfast.quaternions.plot_quaternion_rel_angle': ( 'quaternions.html#plot_quaternion_rel_angle',
                                                                                      'tsfast/quaternions.py'),
                                    'tsfast.quaternions.plot_scalar_inclination': ( 'quaternions.html#plot_scalar_inclination',
                                                                                    'tsfast/quaternions.py'),
                                    'tsfast.quaternions.quatFromAngleAxis': ('quaternions.html#quatfromangleaxis', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.quatFromAngleAxis_np': ( 'quaternions.html#quatfromangleaxis_np',
                                                                                 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.quatInterp': ('quaternions.html#quatinterp', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.quatInterp_np': ('quaternions.html#quatinterp_np', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.rad2deg': ('quaternions.html#rad2deg', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.rand_quat': ('quaternions.html#rand_quat', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.relativeAngle': ('quaternions.html#relativeangle', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.relativeQuat': ('quaternions.html#relativequat', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.relativeQuat_np': ('quaternions.html#relativequat_np', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.rms_inclination': ('quaternions.html#rms_inclination', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.rms_inclination_deg': ( 'quaternions.html#rms_inclination_deg',
                                                                                'tsfast/quaternions.py'),
                                    'tsfast.quaternions.rms_pitch_deg': ('quaternions.html#rms_pitch_deg', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.rms_rel_angle_deg': ('quaternions.html#rms_rel_angle_deg', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.rms_roll_deg': ('quaternions.html#rms_roll_deg', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.rollAngle': ('quaternions.html#rollangle', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.rot_vec': ('quaternions.html#rot_vec', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.safe_acos': ('quaternions.html#safe_acos', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.safe_acos_double': ('quaternions.html#safe_acos_double', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.show_batch': ('quaternions.html#show_batch', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.show_results': ('quaternions.html#show_results', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.smooth_inclination': ( 'quaternions.html#smooth_inclination',
                                                                               'tsfast/quaternions.py')},
            'tsfast.spectogram': { 'tsfast.spectogram.Sequence2Spectrogram': ( 'spectogram.html#sequence2spectrogram',
                                                                               'tsfast/spectogram.py'),
                                   'tsfast.spectogram.Sequence2Spectrogram.__init__': ( 'spectogram.html#sequence2spectrogram.__init__',
                                                                                        'tsfast/spectogram.py'),
                                   'tsfast.spectogram.Sequence2Spectrogram.encodes': ( 'spectogram.html#sequence2spectrogram.encodes',
                                                                                       'tsfast/spectogram.py'),
                                   'tsfast.spectogram.Spectrogram': ('spectogram.html#spectrogram', 'tsfast/spectogram.py'),
                                   'tsfast.spectogram.Spectrogram.__init__': ( 'spectogram.html#spectrogram.__init__',
                                                                               'tsfast/spectogram.py'),
                                   'tsfast.spectogram.Spectrogram.forward': ('spectogram.html#spectrogram.forward', 'tsfast/spectogram.py'),
                                   'tsfast.spectogram.SpectrogramBlock': ('spectogram.html#spectrogramblock', 'tsfast/spectogram.py'),
                                   'tsfast.spectogram.SpectrogramBlock.__init__': ( 'spectogram.html#spectrogramblock.__init__',
                                                                                    'tsfast/spectogram.py'),
                                   'tsfast.spectogram.SpectrogramBlock.from_hdf': ( 'spectogram.html#spectrogramblock.from_hdf',
                                                                                    'tsfast/spectogram.py'),
                                   'tsfast.spectogram.TensorSpectrogram': ('spectogram.html#tensorspectrogram', 'tsfast/spectogram.py'),
                                   'tsfast.spectogram.TensorSpectrogram.show': ( 'spectogram.html#tensorspectrogram.show',
                                                                                 'tsfast/spectogram.py'),
                                   'tsfast.spectogram.TensorSpectrogramInput': ( 'spectogram.html#tensorspectrograminput',
                                                                                 'tsfast/spectogram.py'),
                                   'tsfast.spectogram.TensorSpectrogramOutput': ( 'spectogram.html#tensorspectrogramoutput',
                                                                                  'tsfast/spectogram.py'),
                                   'tsfast.spectogram.complex_norm': ('spectogram.html#complex_norm', 'tsfast/spectogram.py'),
                                   'tsfast.spectogram.spectrogram': ('spectogram.html#spectrogram', 'tsfast/spectogram.py')},
            'tsfast.tune': { 'tsfast.tune.CBRayReporter': ('tune.html#cbrayreporter', 'tsfast/tune.py'),
                             'tsfast.tune.CBRayReporter.after_epoch': ('tune.html#cbrayreporter.after_epoch', 'tsfast/tune.py'),
                             'tsfast.tune.HPOptimizer': ('tune.html#hpoptimizer', 'tsfast/tune.py'),
                             'tsfast.tune.HPOptimizer.__init__': ('tune.html#hpoptimizer.__init__', 'tsfast/tune.py'),
                             'tsfast.tune.HPOptimizer.best_model': ('tune.html#hpoptimizer.best_model', 'tsfast/tune.py'),
                             'tsfast.tune.HPOptimizer.optimize': ('tune.html#hpoptimizer.optimize', 'tsfast/tune.py'),
                             'tsfast.tune.HPOptimizer.optimize_pbt': ('tune.html#hpoptimizer.optimize_pbt', 'tsfast/tune.py'),
                             'tsfast.tune.HPOptimizer.start_ray': ('tune.html#hpoptimizer.start_ray', 'tsfast/tune.py'),
                             'tsfast.tune.HPOptimizer.stop_ray': ('tune.html#hpoptimizer.stop_ray', 'tsfast/tune.py'),
                             'tsfast.tune.LearnerTrainable': ('tune.html#learnertrainable', 'tsfast/tune.py'),
                             'tsfast.tune.LearnerTrainable._export_model': ('tune.html#learnertrainable._export_model', 'tsfast/tune.py'),
                             'tsfast.tune.LearnerTrainable.load_checkpoint': ( 'tune.html#learnertrainable.load_checkpoint',
                                                                               'tsfast/tune.py'),
                             'tsfast.tune.LearnerTrainable.reset_config': ('tune.html#learnertrainable.reset_config', 'tsfast/tune.py'),
                             'tsfast.tune.LearnerTrainable.save_checkpoint': ( 'tune.html#learnertrainable.save_checkpoint',
                                                                               'tsfast/tune.py'),
                             'tsfast.tune.LearnerTrainable.setup': ('tune.html#learnertrainable.setup', 'tsfast/tune.py'),
                             'tsfast.tune.LearnerTrainable.step': ('tune.html#learnertrainable.step', 'tsfast/tune.py'),
                             'tsfast.tune.learner_optimize': ('tune.html#learner_optimize', 'tsfast/tune.py'),
                             'tsfast.tune.log_uniform': ('tune.html#log_uniform', 'tsfast/tune.py'),
                             'tsfast.tune.sample_config': ('tune.html#sample_config', 'tsfast/tune.py'),
                             'tsfast.tune.stop_shared_memory_managers': ('tune.html#stop_shared_memory_managers', 'tsfast/tune.py')}}}
